;;$Id: planck.f90 9 2014-01-31 08:19:35Z awalther $
;;--------------------------------------------------------------------------------------
;; Clouds from AVHRR Extended (CLAVR-x) 1b PROCESSING SOFTWARE Version 5.3
;;
;; NAME: planck.f90 (src)
;;       PLANCK (program)
;;
;; PURPOSE: this module holds the routine to do rapid Planck
;;          computations using a table lookup approach. 
;;
;; DESCRIPTION: This has been shown to speed up CLAVR-x over using the
;;              explicit planck function with expontentials
;;
;; AUTHORS:
;;  Andrew Heidinger, Andrew.Heidinger@noaa.gov
;;
;; COPYRIGHT
;; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;; SUPPORT TO USERS.
;;
;; REVISION HISTORY:
;;   August 31, 2006 - Created
;;
;; ROUTINES:
;; public::
;;   POPULATE_PLANCK_TABLES
;;   PLANCK_RAD_FAST
;;   PLANCK_TEMP_FAST
;;   PLANCK_RAD
;;   PLANCK_TEMP
;;
;;  Modis Band   Avhrr Band   Abi Band   Wavelength  
;;     01            1          2           0.63
;;     02            2          3           0.86
;;     06            3a         5           1.60
;;     20            3b         7           3.75
;;     21            -          7           3.90
;;     22            -          7           3.90
;;     23            -          -           3.95
;;     24            -          -           4.05
;;     25            -          -           4.45
;;     27            -          9           6.70
;;     28            -         10           7.30
;;     29            -         11           8.50
;;     30            -         12           9.70
;;     31            4         14           11.00
;;     32            5         15           12.00
;;     33            -         16           13.30
;;     34            -          -           13.30
;;     35            -          -           13.30
;;     36            -          -           13.30
;;     40            -          -            3.75
;;     41            -          -           11.45
;;
;;--------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------
;; subroutine LOCATE(xx, n, x, j)
;; Numerical recipes bisection search - x will be between xx(j) and xx(j+1) IN FORTRAN
;;--------------------------------------------------------------------------
  pro LOCATE,xx, n, x, j

    ;;Arguments
    ;integer,                        intent(in)  :: n
    ;integer,                        intent(out) :: j
    ;real (kind=ipre),               intent(in)  :: x
    ;real (kind=ipre), dimension(:), intent(in)  :: xx

    ;;Local variables
    ;integer :: i, jl, jm, ju

    jl = 0
    ju = n + 1
    for i = 1, 2*n do begin
       if (ju-jl le 1) then begin
          break ;exit
       endif
       jm = (ju + jl) / 2
       if ((xx(n-1) ge xx(1-1)) eq (x ge xx(jm-1))) then begin
          jl = jm
       endif else begin
          ju = jm
       endelse
    endfor
    if (x eq xx(1-1)) then begin
       j=1 
    endif else begin
       if (x eq xx(n-1)) then begin
          j = n - 1 
       endif else begin
          j = jl 
       endelse	  
    endelse

  end

;;-------------------------------------------------------------------------
 pro COMPUTE_BT_ARRAY,bt,rad,ichan,missing

 ;real(kind=real4), dimension(:,:), intent(in):: rad
 ;integer(kind=int4):: ichan
 ;real(kind=real4):: missing
 ;real(kind=real4), dimension(:,:), intent(out):: bt
 ;integer:: i, j
 ;integer:: nx, ny

  nx = (size(rad,/dimensions))(0)
  ny = (size(rad,/dimensions))(1)

  bt = rad*0.+missing

  for i = 1, nx do begin
    for j = 1, ny do begin
       if (rad(i-1,j-1) ne missing) then begin
          bt(i-1,j-1) = PLANCK_TEMP_FAST(ichan,rad(i-1,j-1))
       endif
    endfor
  endfor

 end 
 
;;-------------------------------------------------------------------------
;; subroutine POPULATE_PLANCK_TABLES(a1_20,a2_20,nu_20,a1_31,a2_31,nu_31,a1_32,a2_32,nu_32)
;;
;; compute planck function tables
;;
;;-------------------------------------------------------------------------
pro POPULATE_PLANCK_TABLES,a1_20,a2_20,nu_20, $
                                   a1_31,a2_31,nu_31, $
                                   a1_32,a2_32,nu_32, $
				   b20,b31,b32


  c1=1.191062d-5
  c2=1.4387863d   
  nplanck = 161
  T_planck_min = 180.0  &  delta_T_planck = 1.0
  B20 = fltarr(nplanck)
  B31 = fltarr(nplanck)
  B32 = fltarr(nplanck)
  T_planck = fltarr(nplanck)

  for i = 1, nplanck do begin
    T_planck(i-1) = T_planck_min + (i-1)*delta_T_planck
    B20(i-1) = c1*(nu_20^3)/(exp((c2*nu_20)/ $
              ((T_planck(i-1)-a1_20)/a2_20))-1.0)
    B31(i-1) = c1*(nu_31^3)/(exp((c2*nu_31)/ $
              ((T_planck(i-1)-a1_31)/a2_31))-1.0)
    B32(i-1) = c1*(nu_32^3)/(exp((c2*nu_32)/ $
              ((T_planck(i-1)-a1_32)/a2_32))-1.0)
  endfor
  

end 
  

;;------------------------------------------------------------------
;; function PLANCK_RAD_FAST(ichan, T, dB_dT) result(B)
;;
;; Subroutine to convert brightness temperature to radiance using a
;; look-up table *Function that returns a scalar*.
;;------------------------------------------------------------------
function PLANCK_RAD_FAST,ichan, T,b20,b31,b32
    ;integer (kind=int4), intent(in) :: ichan
    ;real (kind=real4), intent(in) :: T 
    ;real (kind=real4), optional, intent(out) :: dB_dT
    ;real (kind=real4) :: B, dB_dT_tmp
    ;integer:: l

    ;;--- compute planck emission for cloud temperature
     nplanck = 161
     T_planck_min = 180.0  &  delta_T_planck = 1.0
     l = (T - T_planck_min)/delta_T_planck
     l = 1>((nplanck-1)<l)
    
     T_planck = T_planck_min +findgen(nplanck)*delta_T_planck
   
    case ichan of
    20:  begin
            dB_dT_tmp = (B20(l+1-1)-B20(l-1))/(T_planck(l+1-1)-T_planck(l-1)) 
            B = B20(l-1) + (T - T_planck(l-1)) * (dB_dT_tmp)
         end
    31:  begin
            dB_dT_tmp = (B31(l+1-1)-B31(l-1))/(T_planck(l+1-1)-T_planck(l-1)) 
            B = B31(l-1) + (T - T_planck(l-1)) * (dB_dT_tmp)
         end
    32:  begin
            dB_dT_tmp = (B32(l+1-1)-B32(l-1))/(T_planck(l+1-1)-T_planck(l-1)) 
            B = B32(l-1) + (T - T_planck(l-1)) * (dB_dT_tmp)
         end
    else: print, 'unsupported channel number in Planck Computation, stopping'
    endcase

    dB_dT = dB_dT_tmp

    return, B

end 

;;------------------------------------------------------------------------
;; function PLANCK_TEMP_FAST(ichan, B, dB_dT) result(T)
;;
;; Subroutine to convert radiance (B) to brightness temperature(T) using a
;; look-up table *Function that returns a scalar*.
;;------------------------------------------------------------------
function PLANCK_TEMP_FAST,ichan, B, b20,b31,b32
    ;integer (kind=int4), intent(in) :: ichan
    ;real (kind=real4), intent(in) :: B 
    ;real (kind=real4), optional, intent(out) :: dB_dT
    ;real (kind=real4) :: T, dB_dT_tmp
    ;integer:: l

    nplanck = 161
    T_planck_min = 180.0  &  delta_T_planck = 1.0
     T_planck = T_planck_min +findgen(nplanck)*delta_T_planck
    
    case ichan of
    20:  begin
            locate,B20,nplanck,B,l
	    l = 1>((nplanck-1)<l)
	    dB_dT_tmp = (B20(l+1-1)-B20(l-1))/(T_planck(l+1-1)-T_planck(l-1))
	    T = T_planck(l-1) + (B - B20(l-1)) / (dB_dT_tmp)
         end
    31:  begin
            locate,B31,nplanck,B,l
	    l = 1>((nplanck-1)<l)
	    dB_dT_tmp = (B31(l+1-1)-B31(l-1))/(T_planck(l+1-1)-T_planck(l-1))
	    T = T_planck(l-1) + (B - B31(l-1)) / (dB_dT_tmp)
         end
    32:  begin
            locate,B32,nplanck,B,l
	    l = 1>((nplanck-1)<l)
	    dB_dT_tmp = (B32(l+1-1)-B32(l-1))/(T_planck(l+1-1)-T_planck(l-1))
	    T = T_planck(l-1) + (B - B32(l-1)) / (dB_dT_tmp)
         end
    else: print, 'unsupported channel number in Planck Computation, stopping'
    endcase

    dB_dT = dB_dT_tmp

    return, T

end 

;;----------------------------------------------------------------------
;; function PLANCK_RAD(ichan, T) result(B), not used in the package 
;;----------------------------------------------------------------------

function PLANCK_RAD,ichan, T,a1,a2,nu
    ;integer (kind=int4), intent(in) :: ichan
    ;real (kind=real4), intent(in) :: T 
    ;real (kind=real4) :: B
    c1=1.191062d-5
    c2=1.4387863d   

    case ichan of
    20:  begin
            a1_20=a1 & a2_20=a2 & nu_20=nu
            B = c1*(nu_20^3)/(exp((c2*nu_20)/((T-a1_20)/a2_20))-1.0)
         end
    31:  begin
            a1_31=a1 & a2_31=a2 & nu_31=nu
            B = c1*(nu_31^3)/(exp((c2*nu_31)/((T-a1_31)/a2_31))-1.0)
         end
    32:  begin
            a1_32=a1 & a2_32=a2 & nu_32=nu
            B = c1*(nu_32^3)/(exp((c2*nu_32)/((T-a1_32)/a2_32))-1.0)
         end
    else: print, 'unsupported channel number in Planck Computation, stopping'
    endcase

    return, B

end 
  
;;----------------------------------------------------------------------
;; function PLANCK_TEMP(ichan, B) result(T), not used in the package 
;; otherwise need using c1,c2,un,a1,a2 in the input
;;----------------------------------------------------------------------
  function PLANCK_TEMP,ichan, B,a1,a2,nu
    ;integer (kind=int4), intent(in) :: ichan
    ;real (kind=real4), intent(in) :: B 
    ;real (kind=real4) :: T
    c1=1.191062d-5
    c2=1.4387863d   

    case ichan of
    20:  begin
            a1_20=a1 & a2_20=a2 & nu_20=nu
            T = a1_20 + a2_20 * ((c2*nu_20) / log( 1.0 + (c1*(nu_20^3))/B))
         end
    31:  begin
            a1_31=a1 & a2_31=a2 & nu_31=nu
            T = a1_31 + a2_31 * ((c2*nu_31) / log( 1.0 + (c1*(nu_31^3))/B))
         end
    32:  begin
            a1_32=a1 & a2_32=a2 & nu_32=nu
            T = a1_32 + a2_32 * ((c2*nu_32) / log( 1.0 + (c1*(nu_32^3))/B))
         end
    else: print, 'unsupported channel number in Planck Computation, stopping'
    endcase
    return, T

  end 
