   ;-------------------------------------------------------------------------------------- 
   ;avhrr_caligration_subs_2.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: avhrr_caligration_subs_2.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





;;--------------------------------------------------------------------------
;; compute parameters needed to derive new thermal calibration
;; coefficients using a method similar to that described in the
;; PODUG for HPRT users.  
;;
;; The methodology is 
;;  average all space, blackbody counts and PRT's rom one scan together
;;  average all quantities together for Ncal scanlines
;;  every Ncal scanlines, update a running mean of all these values
;;  the running mean is defined as
;;   new_running_mean = (1-Cal_Smoothing_Weight)*old_running_mean + new_value 
;;
;;  experimentation has shown Ncal = 10 and Cal_Smoothing_Weight = 1/5 result in smooth curves
;;  the variation 1b slope and intercept is approximated with Ncal=5, Cal_Smoothing_Weight = 1.0
;;  unsure if smoother is better (TBD).
;;
;; Author: Andrew Heidinger
;; Date: February 2003
;;
;; Revision - added check for missing space and blackbody counts - noticed
;;           this occured for D95001 - D95005 data for NOAA-14
;;
;;  - modified for one scan at a time processing - akh
;;3

;; Reference:
;; http://www.ncdc.noaa.gov/oa/pod-guide/ncdc/docs/klm/html/c7/sec7-1.htm
;;--------------------------------------------------------------------------
pro COMPUTE_NEW_THERM_CAL_COEF,Line_Idx
 
   @commonblock_1.pro  
   
 iword =0 & Byte_Idx=0 & j=0 & Pix_Idx=0 & Chan_Idx=0 & i4word=0L
 Mean_Space_Count_1_Temp = 0.
 Mean_Space_Count_2_Temp = 0.
 Mean_Space_Count_3_Temp = 0.
 Mean_Space_Count_4_Temp = 0.
 Mean_Space_Count_5_Temp = 0.
 Mean_BB_Count_3_Temp = 0.
 Mean_BB_Count_4_Temp = 0.
 Mean_BB_Count_5_Temp = 0.
 Space_Count_Temp_4_test = 0.
 
 sum_prt = 0
 mean_prt = 0.
 Telemetry_Word   = intarr(103)
 Blackbody_Count  = intarr(5,10)
 space_count_temp = intarr(5,10)
 blackbody_count = fltarr(5,10)
 cal_smoothing_weight = 0.2
 NBB_3b = 0. & NBB_4 = 0. & NBB_5=0.
 
 Valid_Therm_Cal = !sym.true

;;-----------------------------------------------------------------------
;; unpack PRT readings
;;-----------------------------------------------------------------------

;;-- reset prt readings

  Sum_Prt = 0

  ;;-----------------------------------------------------------------------
  ;; unpack telemetry words for AVHRR_KLM_Flag
  ;;-----------------------------------------------------------------------
  if (AVHRR_KLM_Flag eq !sym.YES) then begin
    Byte_Idx = 1091 - 1 
    Sum_Prt = MAKE_I2WORD(Buffer_AVHRR(Byte_Idx:Byte_Idx+1),!sym.SIGNED,Byte_Swap_1b) +  $
              MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+2:Byte_Idx+3),!sym.SIGNED,Byte_Swap_1b) + $
              MAKE_I2WORD(Buffer_AVHRR(Byte_Idx+4:Byte_Idx+5),!sym.SIGNED,Byte_Swap_1b)
  endif else begin
  ;;-----------------------------------------------------------------------
  ;; unpack telemetry words for pre-AVHRR_KLM_Flag
  ;;-----------------------------------------------------------------------
    iword = 1
    for Byte_Idx = 309,448,4 do begin
      i4word = MAKE_I4WORD(Buffer_AVHRR(Byte_Idx-1:Byte_Idx+3-1),!sym.UNSIGNED,Byte_Swap_1b)
      for j = 1,3 do begin
       Telemetry_Word(iword-1) = i4word / (long(1024)^(3-j))
       i4word = i4word - Telemetry_Word(iword-1)*(long(1024)^(3-j))
       iword = iword + 1
       if (iword gt 103) then begin
        break  ;;exit
       endif
      endfor
       if (iword gt 103) then begin
        break  ;;exit
       endif
    endfor

    ;;--- prt readings
    Sum_Prt = Telemetry_Word(18-1)+Telemetry_Word(19-1)+Telemetry_Word(20-1)

  endelse
  
  
  ;;--- increment prt counter
  Prt_Idx = Prt_Idx + 1

  ;;---- check for reset of PRTs (Reset by Prt_Idx = 0 )
  if (Sum_Prt lt 50) then begin
     Prt_Idx = 0
  endif
  
  
  ;;--------------------------------------------------
  ;; test to see if any coefficients can be made
  ;;--------------------------------------------------
  if ((Prt_Idx eq 0) and (Mean_T_Prt le 0.0)) then begin
        print, "Error: PRT Temperatures bad "
        Valid_Therm_Cal = !sym.false
        return
  endif

  ;;---- check to see if out of cycle
  if (Prt_Idx gt 4) then begin
      print, 'Error: PRTs out of order '
      Valid_Therm_Cal = !sym.false
      Prt_Idx = 0    ;; added akh
      return
  endif

  ;;----------------------------------------------------
  ;;---- unpack target view
  ;;----------------------------------------------------
  if (AVHRR_KLM_Flag eq !sym.YES) then begin

    ;;--- the internal target data from AVHRR_KLM_Flag
    for j = 1,10 do begin
      Blackbody_Count(3-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1101+6*(j-1)-1:1102+6*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
      Blackbody_Count(4-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1103+6*(j-1)-1:1104+6*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
      Blackbody_Count(5-1,j-1)  = MAKE_I2WORD(Buffer_AVHRR(1105+6*(j-1)-1:1106+6*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
    endfor
  endif else begin

    ;;--- the internal target data from pre-AVHRR_KLM_Flag
    Chan_Idx = 3
    Pix_Idx = 1
    for iword = 23,52,1 do begin
       Blackbody_Count(Chan_Idx-1, Pix_Idx-1) = Telemetry_Word(iword-1)
       Chan_Idx = Chan_Idx + 1
       if (Chan_Idx gt 5) then begin
          Chan_Idx = 3
          Pix_Idx = Pix_Idx + 1
       endif
     endfor

  endelse

  ;;------------------------------------------------------
  ;;---- unpack space view
  ;;------------------------------------------------------

  if (AVHRR_KLM_Flag eq !sym.YES) then begin

    ;;---space data from AVHRR_KLM_Flag data
    for j = 1,10 do begin
      Space_Count_Temp(1-1,j-1)  = MAKE_I2WORD(Buffer_AVHRR(1161+10*(j-1)-1:1162+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
      Space_Count_Temp(2-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1163+10*(j-1)-1:1164+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
      Space_Count_Temp(3-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1165+10*(j-1)-1:1166+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
      Space_Count_Temp(4-1,j-1) = MAKE_I2WORD(Buffer_AVHRR(1167+10*(j-1)-1:1168+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
      Space_Count_Temp(5-1,j-1)  = MAKE_I2WORD(Buffer_AVHRR(1169+10*(j-1)-1:1170+10*(j-1)-1),!sym.SIGNED,Byte_Swap_1b)
    endfor

  endif else begin

    ;;---space data from pre-AVHRR_KLM_Flag data
    Chan_Idx = 1
    Pix_Idx = 1
    for iword = 53,102,1 do begin
       Space_Count_Temp(Chan_Idx-1, Pix_Idx-1) = Telemetry_Word(iword-1)
       Chan_Idx = Chan_Idx + 1
       if (Chan_Idx gt 5) then begin
          Chan_Idx = 1
          Pix_Idx = Pix_Idx + 1
       endif
    endfor

  endelse
  


   ;;---- check for rapid change in ch4 space count (ie during a moon view)
   
   Space_Count_Temp_4_test = total(Space_Count_Temp(4-1,*))/10.0
   if ((Mean_Space_Count_4 gt 0) and  $
       (abs(Space_Count_Temp_4_test - Mean_Space_Count_4) gt 4.0))  then begin
       print, 'Error: rapid change in ch4 space count '
       Valid_Therm_Cal = !sym.false
       return
   endif
   

   ;;---- check for bad space or blackbody counts
   if ((min(Space_Count_Temp(4-1:5-1,*)) lt 50) or (min(Blackbody_Count(4-1:5-1,*)) lt 50)) then begin
       print, 'Error: ch4 -ch5 space count threshold exceeded '
       Valid_Therm_Cal = !sym.false
       return
   endif

    ;;----- assume a Valid calibration can be made
    if (Valid_Therm_Cal eq !sym.true) then begin
     Cal_Idx = Cal_Idx + 1
    endif

    ;;--- compute blackbody temperature if Valid PRT reading (Eq 7.1.2.4-1 in NOAA KLM User's Guide)
    if ((Prt_Idx gt 0) and (Valid_Therm_Cal eq !sym.true)) then begin
       Mean_Prt = Sum_Prt/3.0   ;;; a float
       T_Prt(Cal_Idx-1) = Prt_Coef(0,Prt_Idx-1) +  $           ;;;;;^^* fortran (0:4,Prt_Idx) bug??? no
                     Prt_Coef(1,Prt_Idx-1)*Mean_Prt + $
                     Prt_Coef(2,Prt_Idx-1)*Mean_Prt^2 + $
                     Prt_Coef(3,Prt_Idx-1)*Mean_Prt^3 + $
                     Prt_Coef(4,Prt_Idx-1)*Mean_Prt^4

    endif

    ;;--- if no PRT reading, fill in with mean value
    if ((Prt_Idx eq 0) and (Valid_Therm_Cal eq !sym.true)) then begin
       T_Prt(Cal_Idx-1) = Mean_T_Prt
    endif

    ;;--- average space and bb counts for this scan
    
    if (Cal_Idx gt 0) then begin
       Space_Count_Temp_1(Cal_Idx-1) = total(Space_Count_Temp(1-1,*))/10.0
       Space_Count_Temp_2(Cal_Idx-1) = total(Space_Count_Temp(2-1,*))/10.0
       Space_Count_Temp_3(Cal_Idx-1) = total(Space_Count_Temp(3-1,*))/10.0
       Space_Count_Temp_4(Cal_Idx-1) = total(Space_Count_Temp(4-1,*))/10.0
       
       Space_Count_Temp_5(Cal_Idx-1) = total(Space_Count_Temp(5-1,*))/10.0

       BB_Count_3(Cal_Idx-1) = total(Blackbody_Count(3-1,*))/10.0
       BB_Count_4(Cal_Idx-1) = total(Blackbody_Count(4-1,*))/10.0
       BB_Count_5(Cal_Idx-1) = total(Blackbody_Count(5-1,*))/10.0
    endif

;;-------------------------------------------------------------------
;; compute running means of space, blackbody and prt temps
;;-------------------------------------------------------------------
    ;;--- if initial spinup period, sum all to get mean
    if (Spinup_New_Therm_Cal eq !sym.true) then begin

      if (Cal_Idx gt 0) then begin
       Mean_T_Prt = total(T_Prt(1-1:Cal_Idx-1))/Cal_Idx                      ;; Ref KLM-POD 7.1.2.4-2
       Mean_Space_Count_1 = total(Space_Count_Temp_1(1-1:Cal_Idx-1))/Cal_Idx
       Mean_Space_Count_2 = total(Space_Count_Temp_2(1-1:Cal_Idx-1))/Cal_Idx
       Mean_Space_Count_3 = total(Space_Count_Temp_3(1-1:Cal_Idx-1))/Cal_Idx
       Mean_Space_Count_4 = total(Space_Count_Temp_4(1-1:Cal_Idx-1))/Cal_Idx
       
       Mean_Space_Count_5 = total(Space_Count_Temp_5(1-1:Cal_Idx-1))/Cal_Idx
       Mean_BB_Count_3 = total(BB_Count_3(1-1:Cal_Idx-1))/Cal_Idx
       Mean_BB_Count_4 = total(BB_Count_4(1-1:Cal_Idx-1))/Cal_Idx
       Mean_BB_Count_5 = total(BB_Count_5(1-1:Cal_Idx-1))/Cal_Idx
      endif else begin
       Mean_T_Prt = !sym.Missing_Value_Real4
       Mean_Space_Count_1 = !sym.Missing_Value_Real4
       Mean_Space_Count_2 = !sym.Missing_Value_Real4
       Mean_Space_Count_3 = !sym.Missing_Value_Real4
       Mean_Space_Count_4 = !sym.Missing_Value_Real4
       Mean_Space_Count_5 = !sym.Missing_Value_Real4
       Mean_BB_Count_3 = !sym.Missing_Value_Real4
       Mean_BB_Count_4 = !sym.Missing_Value_Real4
       Mean_BB_Count_5 = !sym.Missing_Value_Real4
       Valid_Therm_Cal = !sym.false
      endelse

      if (Cal_Idx eq Ncal) then begin
        Spinup_New_Therm_Cal = !sym.false
        Cal_Idx = 0
        T_Prt = T_Prt*0.0
        Space_Count_Temp_1 = Space_Count_Temp_1* 0.0
        Space_Count_Temp_2 = Space_Count_Temp_2* 0.0
        Space_Count_Temp_3 = Space_Count_Temp_3* 0.0
        Space_Count_Temp_4 = Space_Count_Temp_4* 0.0
        Space_Count_Temp_5 = Space_Count_Temp_5* 0.0
        BB_Count_3 = BB_Count_3*0.0
        BB_Count_4 = BB_Count_4*0.0
        BB_Count_5 = BB_Count_5*0.0
      endif

    endif else begin

      ;;--- after Ncal lines, update space and bb count values 
      if (Cal_Idx eq Ncal) then begin

       ;;--- compute running-mean of the black-body temperature
       Mean_T_Prt = (1.0-Cal_Smoothing_Weight)*Mean_T_Prt + Cal_Smoothing_Weight*(total(T_Prt)/Ncal)

       ;;--- compute instantaneous mean of the space counts
       Mean_Space_Count_1_Temp = total(Space_Count_Temp_1)/Ncal
       Mean_Space_Count_2_Temp = total(Space_Count_Temp_2)/Ncal
       Mean_Space_Count_3_Temp = total(Space_Count_Temp_3)/Ncal
       Mean_Space_Count_4_Temp = total(Space_Count_Temp_4)/Ncal
       Mean_Space_Count_5_Temp = total(Space_Count_Temp_5)/Ncal

       ;;--- compute instantaneous mean of the black-body counts
       Mean_BB_Count_3_Temp = total(BB_Count_3)/Ncal
       Mean_BB_Count_4_Temp = total(BB_Count_4)/Ncal
       Mean_BB_Count_5_Temp = total(BB_Count_5)/Ncal

       ;;--- compute running mean of space counts
       Mean_Space_Count_1 = (1.0-Cal_Smoothing_Weight)*Mean_Space_Count_1 + Cal_Smoothing_Weight*Mean_Space_Count_1_Temp
       Mean_Space_Count_2 = (1.0-Cal_Smoothing_Weight)*Mean_Space_Count_2 + Cal_Smoothing_Weight*Mean_Space_Count_2_Temp
       Mean_Space_Count_3 = (1.0-Cal_Smoothing_Weight)*Mean_Space_Count_3 + Cal_Smoothing_Weight*Mean_Space_Count_3_Temp
       Mean_Space_Count_4 = (1.0-Cal_Smoothing_Weight)*Mean_Space_Count_4 + Cal_Smoothing_Weight*Mean_Space_Count_4_Temp
       Mean_Space_Count_5 = (1.0-Cal_Smoothing_Weight)*Mean_Space_Count_5 + Cal_Smoothing_Weight*Mean_Space_Count_5_Temp

       ;;--- compute running mean of black-body counts
       Mean_BB_Count_3 = (1.0-Cal_Smoothing_Weight)*Mean_BB_Count_3 + Cal_Smoothing_Weight*Mean_BB_Count_3_Temp
       Mean_BB_Count_4 = (1.0-Cal_Smoothing_Weight)*Mean_BB_Count_4 + Cal_Smoothing_Weight*Mean_BB_Count_4_Temp
       Mean_BB_Count_5 = (1.0-Cal_Smoothing_Weight)*Mean_BB_Count_5 + Cal_Smoothing_Weight*Mean_BB_Count_5_Temp


       ;;--- handle ch3 differently due to Ch3a/ch3b switching
       ;;--- if a switch detected, reset running means with instantataneous values

       if ((Ch3a_On_AVHRR(Line_Idx-1) eq 1) and (Mean_Space_Count_3 gt 50)) then begin
          Mean_Space_Count_3 = Mean_Space_Count_3_Temp
          Mean_BB_Count_3 =  Mean_BB_Count_3_Temp
       endif

       if ((Ch3a_On_AVHRR(Line_Idx-1) eq 0) and (Mean_Space_Count_3 lt 950)) then begin
          Mean_Space_Count_3 = Mean_Space_Count_3_Temp
          Mean_BB_Count_3 =  Mean_BB_Count_3_Temp
       endif

       Cal_Idx = 0
       T_Prt = T_Prt*0.0
       Space_Count_Temp_1 = Space_Count_Temp_1*0.0
       Space_Count_Temp_2 = Space_Count_Temp_2*0.0
       Space_Count_Temp_3 = Space_Count_Temp_3*0.0
       Space_Count_Temp_4 = Space_Count_Temp_4*0.0
       Space_Count_Temp_5 = Space_Count_Temp_5*0.0
       BB_Count_3 = BB_Count_3*0.0
       BB_Count_4 = BB_Count_4*0.0
       BB_Count_5 = BB_Count_5*0.0

      endif

    endelse


    ;;---  look for space counts that inValid for Ch3a/3b
    if ((Mean_Space_Count_3 gt 50.0) and (Mean_Space_Count_3 lt 950.0)) then begin
             Valid_Therm_Cal = !sym.false
    endif

    if (Valid_Therm_Cal eq !sym.true) then begin

       ;;--- blackbody radiance
       NBB_3b = PLANCK_RAD_FAST(20,Mean_T_Prt,b20,b31,b32)
       NBB_4 = PLANCK_RAD_FAST(31,Mean_T_Prt,b20,b31,b32)
       NBB_5 = PLANCK_RAD_FAST(32,Mean_T_Prt,b20,b31,b32)
       
       ;c1=1.191062d-5
       ;c2=1.4387863d   
       
       ;NBB_3b = c1*nu_20^3/exp(c2*nu_20/Mean_T_Prt-1.)
       ;NBB_4  = c1*nu_31^3/exp(c2*nu_31/Mean_T_Prt-1.)
       ;NBB_5  = c1*nu_32^3/exp(c2*nu_32/Mean_T_Prt-1.)
                     
       ;;--- compute new linear slopes     (Ref 7.1.2.4-5)
       if (Ch3a_On_AVHRR(Line_Idx-1) eq 0) then begin
        IR_Linear_Slope_New(3-1,Line_Idx-1) = (Space_Rad_3b - NBB_3b) / $
                                       (Mean_Space_Count_3 - Mean_BB_Count_3)
        IR_Linear_Intercept_New(3-1,Line_Idx-1) = Space_Rad_3b - $
                  IR_Linear_Slope_New(3-1,Line_Idx-1)*Mean_Space_Count_3
       endif

       
       
       IR_Linear_Slope_New(4-1,Line_Idx-1) = (Space_Rad_4 - NBB_4) / $
                                (Mean_Space_Count_4 - Mean_BB_Count_4)

       IR_Linear_Intercept_New(4-1,Line_Idx-1) = Space_Rad_4 - $
                  IR_Linear_Slope_New(4-1,Line_Idx-1)*Mean_Space_Count_4

       IR_Linear_Slope_New(5-1,Line_Idx-1) = (Space_Rad_5 - NBB_5) / $
                                (Mean_Space_Count_5 - Mean_BB_Count_5)
       IR_Linear_Intercept_New(5-1,Line_Idx-1) = Space_Rad_5 - $
                  IR_Linear_Slope_New(5-1,Line_Idx-1)*Mean_Space_Count_5

     endif else begin
        Valid_Therm_Cal = !sym.false
     endelse

     ;;--- store these into public arrays for access by other routines
     Scan_Space_Counts_Avhrr(1-1,Line_Idx-1) = Mean_Space_Count_1
     Scan_Space_Counts_Avhrr(2-1,Line_Idx-1) = Mean_Space_Count_2
     Scan_Space_Counts_Avhrr(3-1,Line_Idx-1) = Mean_Space_Count_3
     Scan_Space_Counts_Avhrr(4-1,Line_Idx-1) = Mean_Space_Count_4
     Scan_Space_Counts_Avhrr(5-1,Line_Idx-1) = Mean_Space_Count_5
     Scan_BB_Counts_Avhrr(3-1,Line_Idx-1) = Mean_BB_Count_3
     Scan_BB_Counts_Avhrr(4-1,Line_Idx-1) = Mean_BB_Count_4
     Scan_BB_Counts_Avhrr(5-1,Line_Idx-1) = Mean_BB_Count_5
end 



;;-----------------------------------------------------------
;; REFLECTANCE CALIBRATION ROUTINE
;;
;; comments given for channel 1 only but apply to channel 2,3a
;;
;; input
;;   Time_Temp_Since_Launch - the Time_Temp in year_Temps since the launch
;;   Ref_Cal_1b - flag controlling use of 1b coefficients
;;
;;  output
;;   none - only through common memory
;;
;; Parameters defined in 'avhrr_Pixel_common.f' module
;;  Ch1_Gain_low - calibration slope or gain in low count region
;;                  with degradation accounted for
;;  Ch1_Gain_High - calibration slope or gain in high count region
;;                  with degradation accounted for
;;
;; internal parameters
;;  Ref_Ch1_Switch - the albedo at count = count1_Switch
;;
;; arrays used from common memory
;;  Chan_Counts_Avhrr_Temp(1,:,:) - two scanlines of channel 1 counts
;;
;; arrays modified in common memory
;;  Ref_Ch1_Temp - channel 1 albedo for scanline pair, not this is
;;         not corrected for sun-earth distance or for solar zenith angle
;;
;;  variables used from common memory
;;   Ch1_Gain_Low_0 - calibration slope or gain in low count region at launch
;;   Ch1_Gain_High_0 - calibration slope or gain in high count region at launch
;;   Ch1_Switch_Count - count separating low from high count region
;;   Ch1_Dark_Count_Cal - the dark count used for calibration - may differ from the
;;                         the true dark count for any given slope,intercept pair
;;
;; Discussion
;;  This code computes the coefficients to compute the reflectance calibration 
;;  do to the following (Method 1)
;;   R_low = slope_low * (C - C_dark)
;;   R_High = R_Switch + slope_High*(C - C_Switch)
;; 
;; The method in the POD guide is as follows (Method 2)
;;   R_low = Slope_low * C + intercept_low
;;   R_High = Slope_High * C + intercept_High
;;
;; If Ref_Cal_1b is true, then begin the coefficients read in from the 1b 
;; file are converted into forms for Method 1 (done in REF_CAL()).
;; This transformation is exact
;;
;; Revision History
;; Coded January 2003 - A. Heidinger
;;
;; 2004-09-03 - A. Jelenak
;;     Original REF_CAL sub split into two to allow computation of channel gain
;;     coeffs and their storage in the OBS file. REF_CAL_COEFFS sub does this
;;     task now, and REF_CAL just applies them to the Chan_Counts_Avhrr_Temp() array.
;;
;; 2004-12-04 - A. Heidinger
;;     Added capability to use 1b reflectance coeffcients
;;
;; 2008-12-24 - A. Heidinger
;;     Now operates on single scan-line
;;
;; 2014-01-04 - A. Heidinger
;;     Reintegrated REF_CAL_COEFFS into REF_CAL for simplicity
;;
;;
;; Note, if Therm_Cal_1b eq !sym.NO, this routine must be called
;;       after Compute_New_Therm_Cal_Coeffs
;;-----------------------------------------------------------
 pro REF_CAL,Time_Temp_Since_Launch,Line_Idx
   
   ;real(kind=real4), intent(in):: Time_Temp_Since_Launch
   ;integer, intent(in):: Line_Idx
   
   @commonblock_1.pro  
   
   Ch1_Dark_Count_Cal = 0.
   Ch2_Dark_Count_Cal = 0.
   Ch3a_Dark_Count_Cal = 0.
   Ch1_Switch_Count_Cal = 0.
   Ch2_Switch_Count_Cal = 0.
   Ch3a_Switch_Count_Cal = 0.
   Ref_Ch1_Switch = 0.
   Ref_Ch2_Switch = 0.
   Ref_Ch6_Switch = 0.
      
   Valid_Ref_Cal = !sym.true
   

   ;;--- compute coefficient if using level-1b values
   if (Ref_Cal_1b eq !sym.YES) then begin

    Ch1_Gain_low = vis_slope_1(1-1,Line_Idx-1)
    Ch1_Dark_Count_Cal = -1.0*vis_intercept_1(1-1,Line_Idx-1)/vis_slope_1(1-1,Line_Idx-1)
    Ch1_Gain_High = vis_slope_2(1-1,Line_Idx-1)
    Ch1_Switch_Count_Cal = vis_intersection(1-1,Line_Idx-1)

    Ch2_Gain_low = vis_slope_1(2-1,Line_Idx-1)
    Ch2_Dark_Count_Cal = -1.0*vis_intercept_1(2-1,Line_Idx-1)/vis_slope_1(2-1,Line_Idx-1)
    Ch2_Gain_High = vis_slope_2(2-1,Line_Idx-1)
    Ch2_Switch_Count_Cal = vis_intersection(2-1,Line_Idx-1)

    Ch3a_Gain_low = vis_slope_1(3-1,Line_Idx-1)
    Ch3a_Dark_Count_Cal = -1.0*vis_intercept_1(3-1,Line_Idx-1)/vis_slope_1(3-1,Line_Idx-1)
    Ch3a_Gain_High = vis_slope_2(3-1,Line_Idx-1)
    Ch3a_Switch_Count_Cal = vis_intersection(3-1,Line_Idx-1)

    ;;---- compute reflectances at switch count (define consistent with)
    Ref_Ch1_Switch = vis_slope_2(1-1,Line_Idx-1) * vis_intersection(1-1,Line_Idx-1) + vis_intercept_2(1-1,Line_Idx-1)
    Ref_Ch2_Switch = vis_slope_2(2-1,Line_Idx-1) * vis_intersection(2-1,Line_Idx-1) + vis_intercept_2(2-1,Line_Idx-1)
    Ref_Ch6_Switch = vis_slope_2(3-1,Line_Idx-1) * vis_intersection(3-1,Line_Idx-1) + vis_intercept_2(3-1,Line_Idx-1)

   endif else begin

     ;;--- adjust gains to account for degradation
     if (Therm_Cal_1b eq !sym.YES) then begin
       Ch1_Dark_Count_Cal = Ch1_Dark_Count
       Ch2_Dark_Count_Cal = Ch2_Dark_Count
       Ch3a_Dark_Count_Cal = Ch3a_Dark_Count
     endif else begin
        Ch1_Dark_Count_Cal =  Scan_Space_Counts_Avhrr(1-1,Line_Idx-1) 
        Ch2_Dark_Count_Cal = Scan_Space_Counts_Avhrr(2-1,Line_Idx-1)
        Ch3a_Dark_Count_Cal = Scan_Space_Counts_Avhrr(3-1,Line_Idx-1)
     endelse

     ;;-- set switch counts to values in instrument files
     Ch1_Switch_Count_Cal = Ch1_Switch_Count
     Ch2_Switch_Count_Cal = Ch2_Switch_Count
     Ch3a_Switch_Count_Cal = Ch3a_Switch_Count
     
     ;Time_Temp_Since_Launch is a float
     Ch1_Gain_low = Ch1_Gain_Low_0*(100.0+Ch1_Degrad_Low_1*Time_Temp_Since_Launch+ $
                                  Ch1_Degrad_Low_2*Time_Temp_Since_Launch^2)/100.0
     Ch1_Gain_High = Ch1_Gain_High_0*(100.0+Ch1_Degrad_High_1*Time_Temp_Since_Launch+ $
                                  Ch1_Degrad_High_2*Time_Temp_Since_Launch^2)/100.0
     Ch2_Gain_low = Ch2_Gain_Low_0*(100.0+Ch2_Degrad_Low_1*Time_Temp_Since_Launch+ $
                                  Ch2_Degrad_Low_2*Time_Temp_Since_Launch^2)/100.0
     Ch2_Gain_High = Ch2_Gain_High_0*(100.0+Ch2_Degrad_High_1*Time_Temp_Since_Launch+ $
                                  Ch2_Degrad_High_2*Time_Temp_Since_Launch^2)/100.0
     Ch3a_Gain_low = Ch3a_Gain_Low_0*(100.0 + Ch3a_Degrad_Low_1 * Time_Temp_Since_Launch + $
                                  Ch3a_Degrad_Low_2 * Time_Temp_Since_Launch^2)/100.0
     Ch3a_Gain_High = Ch3a_Gain_High_0*(100.0+Ch3a_Degrad_High_1*Time_Temp_Since_Launch + $
                                  Ch3a_Degrad_High_2 * Time_Temp_Since_Launch^2)/100.0

     ;;---- compute reflectances at switch count
     Ref_Ch1_Switch = Ch1_Gain_low * (Ch1_Switch_Count_Cal - Ch1_Dark_Count_Cal)
     Ref_Ch2_Switch = Ch2_Gain_low * (Ch2_Switch_Count_Cal - Ch2_Dark_Count_Cal)
     Ref_Ch6_Switch = Ch3a_Gain_low * (Ch3a_Switch_Count_Cal - Ch3a_Dark_Count_Cal)

     ;;--- test for Valid dark count
     if (abs(Ch1_Dark_Count_Cal - Ch1_Dark_Count) gt 5) then begin
	;stop
        Valid_Ref_Cal = !sym.false
     endif

   endelse

   ;;---- determine if coefficients are Valid
   if ((Ch1_Gain_low lt 0.0) or (Ch1_Gain_High lt 0.0)) then begin
        Valid_Ref_Cal = !sym.false
   endif

   ;;--- Apply Calibration  ;; for ch struct, see pixel_common.f90
   if (Valid_Ref_Cal eq !sym.true) then begin
   
   
     ;;--- channel 1
     xxx = where(Chan_Counts_Avhrr(1-1,*,Line_Idx-1) le Ch1_Switch_Count_Cal, cxx)
     if cxx gt 0 then ch(1-1).Ref_Toa(xxx,Line_Idx-1) = $
        Ch1_Gain_low*(Chan_Counts_Avhrr(1-1,xxx,Line_Idx-1) - Ch1_Dark_Count_Cal)
     xxx = where(Chan_Counts_Avhrr(1-1,*,Line_Idx-1) gt Ch1_Switch_Count_Cal, cxx)
     if cxx gt 0 then ch(1-1).Ref_Toa(xxx,Line_Idx-1) = $
        Ref_Ch1_Switch + Ch1_Gain_High*(Chan_Counts_Avhrr(1-1,xxx,Line_Idx-1) - Ch1_Switch_Count_Cal)

     ;;--- channel 2
     xxx = where(Chan_Counts_Avhrr(2-1,*,Line_Idx-1) le Ch2_Switch_Count_Cal, cxx)
     if cxx gt 0 then ch(2-1).Ref_Toa(xxx,Line_Idx-1) = $
        Ch2_Gain_low*(Chan_Counts_Avhrr(2-1,xxx,Line_Idx-1) - Ch2_Dark_Count_Cal)
     xxx = where(Chan_Counts_Avhrr(2-1,*,Line_Idx-1) gt Ch2_Switch_Count_Cal, cxx)
     if cxx gt 0 then ch(2-1).Ref_Toa(xxx,Line_Idx-1) = $
        Ref_Ch2_Switch + Ch2_Gain_High*(Chan_Counts_Avhrr(2-1,xxx,Line_Idx-1) - Ch2_Switch_Count_Cal)

     ;;--- channel 3a
     if (Ch3a_On_AVHRR(Line_Idx-1) eq !sym.NO) then begin
        ;ch(6).Ref_Toa(*,Line_Idx-1) = Missing_Value_Real4 
        ch(3-1).Ref_Toa(*,Line_Idx-1) = !sym.Missing_Value_Real4 
     endif else begin
        xxx = where(Chan_Counts_Avhrr(3-1,*,Line_Idx-1) le Ch3a_Switch_Count_Cal, cxx)
        if cxx gt 0 then   $
	   ;ch(6).Ref_Toa(xxx,Line_Idx-1) = $
	   ch(3-1).Ref_Toa(xxx,Line_Idx-1) = $
	      Ch3a_Gain_low*(Chan_Counts_Avhrr(3-1,xxx,Line_Idx-1) - Ch3a_Dark_Count_Cal)
        xxx = where(Chan_Counts_Avhrr(3-1,*,Line_Idx-1) gt Ch3a_Switch_Count_Cal, cxx)
        if cxx gt 0 then   $	
           ;ch(6).Ref_Toa(xxx,Line_Idx-1) = $
           ch(3-1).Ref_Toa(xxx,Line_Idx-1) = $
	      Ref_Ch6_Switch + Ch3a_Gain_High*(Chan_Counts_Avhrr(3-1,xxx,Line_Idx-1) - Ch3a_Switch_Count_Cal) 
     endelse
     
     
     

   endif else begin
      ch(1-1).Ref_Toa(*,Line_Idx-1) = !sym.Missing_Value_Real4 
      ch(2-1).Ref_Toa(*,Line_Idx-1) = !sym.Missing_Value_Real4 
      ch(3-1).Ref_Toa(*,Line_Idx-1) = !sym.Missing_Value_Real4 
   endelse
   

end

