   ;-------------------------------------------------------------------------------------- 
   ;avhrr_caligration_subs_4.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: avhrr_caligration_subs_4.pro
   ; PURPOSE: subroutine to extract, and calibrate AVHRR GAC data
   ; DESCRIPTION: subroutine to calibrate AVHRR Global Area Coverage (GAC).
   ;              The calibration approach is developed by Andy Heidinger (Andrew.Heidinger@noaa.gov). 
   ;              It was in Fortran language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ;   
   ;-------------------------------------------------------------------------------------- 





;;-----------------------------------------------------------
;; THERMAL CALIBRATION
;;
;; variables
;;    Therm_Cal_1b - use 1b cal. coefficients (true)
;;    AVHRR_KLM_Flag - logical flag set to true if this AVHRR_KLM_Flag data
;;    Ch3a_On_AVHRR_Temp - integer flag set to 0 if Ch3a is off
;;    count3 - vector of channel 3 counts
;;    count4 - vector of channel 4 counts
;;    count5 - vector of channel 5 counts
;;    d_coef - the "d" coefficient as illustrated in AVHRR_KLM_Flag-POD
;;    a_coef - the "a" coefficient as illustrated in AVHRR_KLM_Flag-POD
;;    b_coef - the "b" coefficient as illustrated in AVHRR_KLM_Flag-POD
;;    linear_slope - slope in linear radiance computation
;;    linear_intercept - intercept in linear radiance computation
;;    nu_20, a1_20, a2_20 - radiance to temperature conversion parameters
;;                        (see AVHRR_KLM_Flag-PODUG for example)
;;    nu_31, a1_31, a2_31 - radiance to temperature conversion parameters
;;    nu_32, a1_32, a2_32 - radiance to temperature conversion parameters
;;
;;
;; output 
;;   Rad_Ch20_Temp, Rad_Ch31_Temp, Rad_Ch32_Temp - vector of channel radiance in mW/m^2/str/cm^-1
;;   Bt_Ch20_Temp, Bt_Ch31_Temp, Bt_Ch32_Temp - vector of brightness temperature in K
;;
;; Some Notes
;; 1- pre-AVHRR_KLM_Flag data is first converted to a radiance assuming a linear calibration
;;    then begin the non-linear adjustment is made
;; 2- Does not incorporate the correct procedure for data
;;    before NOAA-14.  Need to review original CLAVR-1 for this.
;; 3 - AVHRR_KLM_Flag data, the non-linear correction is added to the linear radiance 
;;     but for non-AVHRR_KLM_Flag data is the non-linear correction is additive (see pods)
;;     this is reflected in the difference is the pre-AVHRR_KLM_Flag and AVHRR_KLM_Flag coefs.
;;
;; Revision History
;;  Coded January 2003 - A. Heidinger
;;        February 2003 - added Therm_Cal_1b argument and new calibration
;;
;;
;; some clarification
;;  the AVHRR_KLM_Flag-POD nomenclature has changed.  
;;
;;---------------------------------------------------------------------------
pro THERM_CAL,Line_Idx_Min_Segment,Num_Scans_Read

  ;;;ch(20)-->ch(4); ch(31)-->ch(5); ch(32)-->ch(6)
  
  ;integer, intent(in):: Num_Scans_Read
  ;integer(kind=int4):: i
  ;integer(kind=int4):: j
  ;integer, parameter:: dx = 2
  ;integer, parameter:: dy = 0
  ;integer:: Num_Valid
  ;integer:: Elem_Idx_Min
  ;integer:: Elem_Idx_Max
  ;integer:: Line_Idx_Max
  ;integer:: Line_Idx_Min
  ;integer:: Elem_Idx
  ;integer:: Line_Idx
  ;integer, parameter:: Ch20_Counts_Filter_Thresh = 900 
  
   @commonblock_1.pro  
   
  i=0L & j=0L & dx=2 & dy=0 & Num_Valid=0
  Elem_Idx_Min=0 & Elem_Idx_Max=0 & Line_Idx_Max=0 & Line_Idx_Min=0
  Elem_Idx=0 & Line_Idx=0
  Ch20_Counts_Filter_Thresh = 900
      
  ;;------------------------------------------------------------------------
  ;; Perform Filter on Ch20 counts for appropriate sensors
  ;; Do this only for TIROS-N, NOAA-5,6,7,8,9,10
  ;;------------------------------------------------------------------------
  if ((Sc_Id_WMO ge 706 and Sc_Id_WMO le 708) or (Sc_Id_WMO ge 200 and Sc_Id_WMO le 202)) then begin

       Temp_Pix_Array = reform(Chan_Counts_Avhrr(3-1,*,*))
       Ch20_Counts_Filtered = reform(Chan_Counts_Avhrr(3-1,*,*))
       One_Byte_Temp = byte(reform(Temp_Pix_Array))*0b+1b
       xxx = where(Temp_Pix_Array le 100 or Temp_Pix_Array gt 1024,cxx)
       if cxx gt 0 then One_Byte_Temp(xxx) = 0b

       for Elem_Idx = 1, Num_Pix do begin

          Elem_Idx_Min = Num_Pix<(1>(Elem_Idx - dx))
          Elem_Idx_Max = Num_Pix<(1>(Elem_Idx + dx))

          for Line_Idx = 1, Num_Scans_Read do begin  

             if (Chan_Counts_Avhrr(3-1,Elem_Idx-1,Line_Idx-1) gt Ch20_Counts_Filter_Thresh) then begin
                Line_Idx_Min = Num_Scans_Read<(1>(Line_Idx - dy))
                Line_Idx_Max = Num_Scans_Read<(1>(Line_Idx + dy))
                Num_Valid = total(One_Byte_Temp(Elem_Idx_Min-1:Elem_Idx_Max-1,Line_Idx_Min-1:Line_Idx_Max-1))
                if (Num_Valid gt 1) then begin
                   Ch20_Counts_Filtered(Elem_Idx-1,Line_Idx-1) =  $
                        total(One_Byte_Temp(Elem_Idx_Min-1:Elem_Idx_Max-1,Line_Idx_Min-1:Line_Idx_Max-1)* $
                        Temp_Pix_Array(Elem_Idx_Min-1:Elem_Idx_Max-1,Line_Idx_Min-1:Line_Idx_Max-1)) / Num_Valid
                endif
             endif
          endfor
       endfor
       Chan_Counts_Avhrr(3-1,*,*) = fix(Ch20_Counts_Filtered)

  endif 

  ;;------------------------------------------------------------------------
  ;; Loop through each line and calibrate
  ;;------------------------------------------------------------------------
  for j = Line_Idx_Min_Segment, Line_Idx_Min_Segment + Num_Scans_Read - 1  do begin

    if (Bad_Scan_Flag(j-1) eq !sym.YES) then begin
       continue        ;;cycle
    endif 
    ;;----  convert counts to radiance
    if (Therm_Cal_1b eq !sym.YES) then begin

       if (AVHRR_KLM_Flag eq !sym.YES) then begin       ;;apply level-1b non-linear correction

         if (Ch3a_On_AVHRR(j-1) eq 0 and Chan_On_Flag_Default(20-1) eq !sym.YES) then begin
           ch(4-1).Rad_Toa(*,j-1) = ir_coef_1_1b(3-1,j-1) + ir_coef_2_1b(3-1,j-1)*Chan_Counts_Avhrr(3-1,*,j-1) +  $
                        ir_coef_3_1b(3-1,j-1)*long(Chan_Counts_Avhrr(3-1,*,j-1))^2
         endif
         if (Chan_On_Flag_Default(31-1) eq !sym.YES) then begin
             ch(5-1).Rad_Toa(*,j-1) = ir_coef_1_1b(4-1,j-1) + ir_coef_2_1b(4-1,j-1)*Chan_Counts_Avhrr(4-1,*,j-1) +  $
                       ir_coef_3_1b(4-1,j-1)*long(Chan_Counts_Avhrr(4-1,*,j-1))^2
         endif
         if (Chan_On_Flag_Default(32-1) eq !sym.YES) then begin
            ch(6-1).Rad_Toa(*,j-1) = ir_coef_1_1b(5-1,j-1) + ir_coef_2_1b(5-1,j-1)*Chan_Counts_Avhrr(5-1,*,j-1) +  $
                            ir_coef_3_1b(5-1,j-1)*long(Chan_Counts_Avhrr(5-1,*,j-1))^2
         endif

       endif else begin                         ;;compute level-1b linear correction for pre-AVHRR_KLM_Flag

         ;;--- Eqn 7.1.2.4-7
         if (Chan_On_Flag_Default(20-1) eq !sym.YES) then begin
           ch(4-1).Rad_Toa(*,j-1) = ir_linear_slope_1b(3-1,j-1)*Chan_Counts_Avhrr(3-1,*,j-1) + ir_linear_intercept_1b(3-1,j-1)
         endif
         if (Chan_On_Flag_Default(31-1) eq !sym.YES) then begin
           ch(5-1).Rad_Toa(*,j-1) = ir_linear_slope_1b(4-1,j-1)*Chan_Counts_Avhrr(4-1,*,j-1) + ir_linear_intercept_1b(4-1,j-1)
         endif
         if (Chan_On_Flag_Default(32-1) eq !sym.YES) then begin
           ch(6-1).Rad_Toa(*,j-1) = ir_linear_slope_1b(5-1,j-1)*Chan_Counts_Avhrr(5-1,*,j-1) + ir_linear_intercept_1b(5-1,j-1)
         endif

       endelse

    endif else begin    ;;generate linear radiance using internally computed coefficients

      if (Ch3a_On_AVHRR(j-1) eq 0 and Chan_On_Flag_Default(20-1) eq !sym.YES) then begin
         ch(4-1).Rad_Toa(*,j-1) = IR_Linear_Slope_New(3-1,j-1)*Chan_Counts_Avhrr(3-1,*,j-1) + IR_Linear_Intercept_New(3-1,j-1)
      endif
      if (Chan_On_Flag_Default(31-1) eq !sym.YES) then begin
        ch(5-1).Rad_Toa(*,j-1) = IR_Linear_Slope_New(4-1,j-1)*Chan_Counts_Avhrr(4-1,*,j-1) + IR_Linear_Intercept_New(4-1,j-1)
      endif
      if (Chan_On_Flag_Default(32-1) eq !sym.YES) then begin
        ch(6-1).Rad_Toa(*,j-1) = IR_Linear_Slope_New(5-1,j-1)*Chan_Counts_Avhrr(5-1,*,j-1) + IR_Linear_Intercept_New(5-1,j-1)
      endif
    endelse

   ;;--- apply non-linear correction where appropriate (non-AVHRR_KLM_Flag or any if thermal cal redone)
   if ((AVHRR_KLM_Flag eq !sym.NO) or (Therm_Cal_1b eq !sym.NO)) then begin 

     if (Ch3a_On_AVHRR(j-1) eq 0 and Chan_On_Flag_Default(20-1) eq !sym.YES) then begin
       ch(4-1).Rad_Toa(*,j-1) = ch(4-1).Rad_Toa(*,j-1) + b0_3b + b1_3b*ch(4-1).Rad_Toa(*,j-1) + b2_3b*ch(4-1).Rad_Toa(*,j-1)^2
     endif
     if (Chan_On_Flag_Default(31-1) eq !sym.YES) then begin
       ch(5-1).Rad_Toa(*,j-1) = ch(5-1).Rad_Toa(*,j-1) + b0_4 + b1_4*ch(5-1).Rad_Toa(*,j-1) + b2_4*ch(5-1).Rad_Toa(*,j-1)^2
     endif
     if (Chan_On_Flag_Default(32-1) eq !sym.YES) then begin
       ch(6-1).Rad_Toa(*,j-1) = ch(6-1).Rad_Toa(*,j-1) + b0_5 + b1_5*ch(6-1).Rad_Toa(*,j-1) + b2_5*ch(6-1).Rad_Toa(*,j-1)^2
     endif
   endif
  endfor
  
   ;;---- convert radiance to brightness temperature
    
  c1=1.191062d-5
  c2=1.4387863d   
  
  if (Chan_On_Flag_Default(20-1) eq !sym.YES) then begin
     xxx = where(ch(4-1).Rad_Toa(*,*) gt 0.0, cxx)
     ;if cxx gt 0 then ch(4-1).Bt_Toa(xxx) = c2*nu_20/alog(c1*nu_20^3/ch(4-1).Rad_Toa(xxx)+1.)
     if cxx gt 0 then ch(4-1).Bt_Toa(xxx) = (c2*nu_20/alog(c1*nu_20^3/ch(4-1).Rad_Toa(xxx)+1.))*a2_20+a1_20
     xxx = where(ch(4-1).Rad_Toa(*,*) le 0.0, cxx)
     if cxx gt 0 then ch(4-1).Bt_Toa(xxx) =  !sym.Missing_Value_Real4
  endif else begin
          ch(4-1).Rad_Toa(*,*) = !sym.Missing_Value_Real4
          ch(4-1).Bt_Toa(*,*) = !sym.Missing_Value_Real4
  endelse    
     
  ;;; in Andy's approach, T is first converted to T*(7.1.2.4-3), then radiance is calculated based on T*.
  ;;; when T is solved from radiance, solved-T is in the T space, not T*. so, no furhter conversion is needed.
  ;;; while in the approach below, solved-T is in the T* space, and further conversion is needed.    
     
  if (Chan_On_Flag_Default(31-1) eq !sym.YES) then begin
     xxx = where(ch(5-1).Rad_Toa(*,*) gt 0.0, cxx)
     ;if cxx gt 0 then ch(5-1).Bt_Toa(xxx) = c2*nu_31/alog(c1*nu_31^3/ch(5-1).Rad_Toa(xxx)+1.)
     if cxx gt 0 then ch(5-1).Bt_Toa(xxx) = (c2*nu_31/alog(c1*nu_31^3/ch(5-1).Rad_Toa(xxx)+1.))*a2_31+a1_31
     xxx = where(ch(5-1).Rad_Toa(*,*) le 0.0, cxx)
     if cxx gt 0 then ch(5-1).Bt_Toa(xxx) =  !sym.Missing_Value_Real4
  endif else begin
          ch(5-1).Rad_Toa(*,*) = !sym.Missing_Value_Real4
          ch(5-1).Bt_Toa(*,*) = !sym.Missing_Value_Real4
  endelse    
  
  if (Chan_On_Flag_Default(32-1) eq !sym.YES) then begin
     xxx = where(ch(6-1).Rad_Toa(*,*) gt 0.0, cxx)
     ;if cxx gt 0 then ch(6-1).Bt_Toa(xxx) = c2*nu_32/alog(c1*nu_32^3/ch(6-1).Rad_Toa(xxx)+1.)
     if cxx gt 0 then ch(6-1).Bt_Toa(xxx) = (c2*nu_32/alog(c1*nu_32^3/ch(6-1).Rad_Toa(xxx)+1.))*a2_32+a1_32
     xxx = where(ch(6-1).Rad_Toa(*,*) le 0.0, cxx)
     if cxx gt 0 then ch(6-1).Bt_Toa(xxx) =  !sym.Missing_Value_Real4
  endif else begin
          ch(6-1).Rad_Toa(*,*) = !sym.Missing_Value_Real4
          ch(6-1).Bt_Toa(*,*) = !sym.Missing_Value_Real4
  endelse    
 
  
  ;;----- additional quality check on Bt_Ch31
  for j = Line_Idx_Min_Segment, Line_Idx_Min_Segment + Num_Scans_Read - 1 do begin
    if (min(ch(5-1).Bt_Toa(*,j-1)) lt 0.0) then begin
      Bad_Scan_Flag(j-1) = !sym.YES
    endif
  endfor

end 







;;-----------------------------------------------------------
;; this converts the dual gain reflectance counts to single
;; gain for the AVHRR_KLM_Flag sensors
;;-----------------------------------------------------------
pro CONVERT_AVHRR_COUNTS_SINGLE_GAIN,j1,j2
 ;integer, intent(in):: j1,j2
 ;integer, intent(in):: AVHRR_KLM_Flag
 ;integer:: i, j
 
   @commonblock_1.pro  
   

 if (Chan_On_Flag_Default(1-1) eq !sym.NO) then return
 if (Chan_On_Flag_Default(2-1) eq !sym.NO) then return
 if (Chan_On_Flag_Default(6-1) eq !sym.NO) then return

 if (AVHRR_KLM_Flag eq !sym.NO) then begin

  Chan_Counts_Avhrr_Sg(*,*,j1-1:j2-1) = Chan_Counts_Avhrr(*,*,j1-1:j2-1)
  for j = j1,j1+j2-1 do begin
    Chan_Counts_Avhrr_Sg(1-1,*,j-1) = Chan_Counts_Avhrr_Sg(1-1,*,j-1) - Scan_Space_Counts_Avhrr(1-1,j-1)
    Chan_Counts_Avhrr_Sg(2-1,*,j-1) = Chan_Counts_Avhrr_Sg(2-1,*,j-1) - Scan_Space_Counts_Avhrr(2-1,j-1)
    if (Ch3a_On_AVHRR(j-1) eq !sym.YES) then begin
      Chan_Counts_Avhrr_Sg(3-1,*,j-1) = Chan_Counts_Avhrr_Sg(3-1,*,j-1) - Scan_Space_Counts_Avhrr(3-1,j-1)
    endif else begin
      Chan_Counts_Avhrr_Sg(3-1,*,j-1) = !sym.Missing_Value_Int2
    endelse
  endfor

 endif else begin

  for j = j1, j1+j2-1 do begin

   ;;--- check for bad scans (note Bad_Pixel_Mask does yet exist)
   if (Bad_Scan_Flag(j-1) eq !sym.YES) then begin
        continue ;cycle
   endif

   for i = 1, Num_Pix do begin

     ;;--- channel 1
     if (Chan_On_Flag_Default(1-1) eq !sym.YES) then begin
      if (Chan_Counts_Avhrr(1-1,i-1,j-1) lt Ch1_Switch_Count) then begin
         Chan_Counts_Avhrr_Sg(1-1,i-1,j-1) = Scan_Space_Counts_Avhrr(1-1,j-1) + 0.5*(Chan_Counts_Avhrr(1-1,i-1,j-1) - Scan_Space_Counts_Avhrr(1-1,j-1))
      endif else begin
         Chan_Counts_Avhrr_Sg(1-1,i-1,j-1) = (Scan_Space_Counts_Avhrr(1-1,j-1) + 0.5*(Ch1_Switch_Count - Scan_Space_Counts_Avhrr(1-1,j-1))) + $
                                 3.0*(Chan_Counts_Avhrr(1-1,i-1,j-1) - Ch1_Switch_Count)/2.0
      endelse
       Chan_Counts_Avhrr_Sg(1-1,i-1,j-1) = Chan_Counts_Avhrr_Sg(1-1,i-1,j-1) - Scan_Space_Counts_Avhrr(1-1,j-1)
     endif

     ;;--- channel 2
     if (Chan_On_Flag_Default(2-1) eq !sym.YES) then begin
      if (Chan_Counts_Avhrr(2-1,i-1,j-1) lt Ch2_Switch_Count) then begin
       Chan_Counts_Avhrr_Sg(2-1,i-1,j-1) = Scan_Space_Counts_Avhrr(2-1,j-1) + 0.5*(Chan_Counts_Avhrr(2-1,i-1,j-1) - Scan_Space_Counts_Avhrr(2-1,j-1))
      endif else begin
       Chan_Counts_Avhrr_Sg(2-1,i-1,j-1) = (Scan_Space_Counts_Avhrr(2-1,j-1) + 0.5*(Ch2_Switch_Count - Scan_Space_Counts_Avhrr(2-1,j-1))) + $
                                 3.0*(Chan_Counts_Avhrr(2-1,i-1,j-1) - Ch2_Switch_Count)/2.0
      endelse
      Chan_Counts_Avhrr_Sg(2-1,i-1,j-1) = Chan_Counts_Avhrr_Sg(2-1,i-1,j-1) - Scan_Space_Counts_Avhrr(1-1,j-1)
    endif

    ;;--- channel 3a
    if (Chan_On_Flag_Default(6-1) eq !sym.YES) then begin

     if (Ch3a_On_AVHRR(j-1) eq !sym.YES) then begin

        if (Chan_Counts_Avhrr(3-1,i-1,j-1) lt Ch3a_Switch_Count) then begin
           Chan_Counts_Avhrr_Sg(3-1,i-1,j-1) = Scan_Space_Counts_Avhrr(3-1,j-1) + 0.25*(Chan_Counts_Avhrr(3-1,i-1,j-1) - Scan_Space_Counts_Avhrr(3-1,j-1))
        endif else begin
           Chan_Counts_Avhrr_Sg(3-1,i-1,j-1) = (Scan_Space_Counts_Avhrr(3-1,j-1) + 0.25*(Ch3a_Switch_Count - Scan_Space_Counts_Avhrr(3-1,j-1))) + $
                                 1.75*(Chan_Counts_Avhrr(3-1,i-1,j-1) - Ch3a_Switch_Count)
        endelse

        Chan_Counts_Avhrr_Sg(3-1,i-1,j-1) = Chan_Counts_Avhrr_Sg(3-1,i-1,j-1) - Scan_Space_Counts_Avhrr(1-1,j-1)

     endif else begin

        Chan_Counts_Avhrr_Sg(3-1,i-1,j-1) = !sym.Missing_Value_Int2

     endelse

    endif

    endfor
   endfor

 endelse

 ;Ch1_Counts = Chan_Counts_Avhrr_Sg(1-1,*,*)
 ;Ch2_Counts = Chan_Counts_Avhrr_Sg(2-1,*,*)
 ;Ch6_Counts = Chan_Counts_Avhrr_Sg(3-1,*,*)

end 
