   ;-------------------------------------------------------------------------------------- 
   ;composite_lst.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: composite_lst.pro
   ; PURPOSE: make a composite of AVHRR channel, viewing angle,and UTC data
   ; DESCRIPTION: subroutine to composite the AVHRR channel, viewing angle,and UTC data to
   ;              5 km EASE-Grid in the polar regions. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutines in this file
   ; 
   ; Data file used 
   ; t5n_corr.dat  t5s_corr.dat  
   ; input:
   ;  (BT: Brightness Temperature)
   ;  filenaem.N1.1: ch1 reflecance of first part of the overpss before passing north pole point in the Northen Hemisphere
   ;  filenaem.N1.2: ch2 reflecance of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.3: ch3a reflecance/ch3b BT of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.4: ch4 BT of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.5: ch5 BT of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.SZA: solar zenith angle in the NH
   ;  filenaem.N1.ELEV: sensor elevation angle in the NH
   ;  filenaem.N1.AZIM: sensor relative azimuth angle in the NH
   ;  filenaem.N1.TIME: time of observation in UTC in the NH
   ;  filenaem.N1.SAZA: satellite relative azimuth angle in the NH
   ;  filenaem.N2.* :  second part of the overpss after passing north pole point in the NH
   ;  filenaem.S1.* : first part of the overpss before passing south pole point in the Sourthern Hemisphere 
   ;  filenaem.S2.* : second part of the overpss after passing south pole point in the Sourthern Hemisphere 
   ; output:
   ; app_n005_####_***_0400.img: APP daily composite at 0400 solar local time on julian day *** in year #### in the Arctic
   ; app_n005_####_***_1400.img: APP daily composite at 1400 solar local time on julian day *** in year #### in the Arctic
   ; app_s005_####_***_0200.img: APP daily composite at 0200 solar local time on julian day *** in year #### in the Antarctic
   ; app_s005_####_***_1400.img: APP daily composite at 1400 solar local time on julian day *** in year #### in the Antarctic
   ;-------------------------------------------------------------------------------------- 





forward_function ch3a_active

pro composite_lst,flns,year,jday,satnum
;---------------------------------------------------------------------------
; This program composites NOAA-16 AVHRR GAC data collected throughout a day 
; into two images per day at local solar time lst for each polar region.  
; The input
; images are from the CCAR 'navigate' program.
;
; Input:
;   flns   : An array of file names of the GAC data.  Each name is the 
;            root of the file name without the ".1", ".SZA", etc. extension
;            but with the hemisphere label (".N1", ".N2", and ".S").
;   year   : 4-digit year
;   jday   : julian day
;   lst    : local solar time when the composite is 
;   satnum  : satellite number
;
; Output files:
;   Two images with the names ""north.lst.##.img" and "south.lst.##.img" for the northern 
;   and southern polar regions at lcoal solar time lst.
;
;  Calls:
;    read_merge
;    drop_in
;    ch3a_active
;---------------------------------------------------------------------------

; Output file names
    lst_ar1 = 4
    lst_ar2 = 14
    lst_an1 = 2
    lst_an2 = 14
    Nfile1 = 'app_n005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_ar1,format='(i02)')+'00.img'
    Nfile2 = 'app_n005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_ar2,format='(i02)')+'00.img'
    Sfile1 = 'app_s005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_an1,format='(i02)')+'00.img'
    Sfile2 = 'app_s005_'+string(year,format='(i04)')+string(jday,format='(i03)')+'_'+string(lst_an2,format='(i02)')+'00.img'
    
    str_pos1 = 15
    str_len1 = 3
    str_pos2 = 20
    str_len2 = 2
    str_pos3 = 26
    str_len3 = 2


; Define the output images
; Arctic
;************ data file used to convert GMT (UTC) to mlocal solar time ********************
time_corr_fn = '../t5n_corr.dat' ;'t5n_corr.dat'          ;;arctic
ncols = 1805 & nrows = 1805  & time_corr = fltarr(ncols,nrows)
openr,ur,time_corr_fn,/get_lun
      readu,ur,time_corr
close,ur
;************ convert GMT (UTC) to mlocal solar time ********************

dimxyN = 1805
nfields = 9
img1 = intarr(dimxyN,dimxyN,nfields)
img2 = intarr(dimxyN,dimxyN,nfields)
score1 = fltarr(dimxyN,dimxyN) + 100.   ; Scan-time score. 0=best
score2 = fltarr(dimxyN,dimxyN) + 100.   ; Scan-time score. 0=best

; READ the images one at a time and process them, first the northern 
; hemisphere (two sets of images; see 'extract') then the southern.

; Determine if AVHRR channel 3A is used in the sunlit portions

;*************************************************************************
satnum = satnum   
caldat,julday(1,1,year)+jday-1,mo,dy,yy
use_ch3a = ch3a_active(satnum,year,mo,dy)
;***************************need address this ****************************

; Northern  hemisphere
satjday = 0
hour_start = 0
hour_end   = 0

iwhere = where(strmatch(flns,'*.N1') eq 1,count)  ; set 1
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end
  if hour_end gt hour_start then flag=0 else flag=1  
  
  ncols = dimxyN
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  drop_in,img,img1,score1,lst_ar1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_ar2,hourdiff,flag,time_corr
  ;drop_in,hemis,ncols,nrows,img,n12z,n12zscore,lst,hourdiff,flag  
endfor

iwhere = where(strmatch(flns,'*.N2') eq 1,count)  ; set 2
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end
  if hour_end gt hour_start then flag=0 else flag=1
  
  ncols = dimxyN
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  drop_in,img,img1,score1,lst_ar1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_ar2,hourdiff,flag,time_corr
  ;drop_in,hemis,ncols,nrows,img,n12z,n12zscore,lst,hourdiff,flag
endfor
;  WRITE the composites.
openw,u,Nfile1,/get_lun
writeu,u,img1
free_lun,u

openw,u,Nfile2,/get_lun
writeu,u,img2
free_lun,u


; Southern  hemisphere
;Antarctica
;************ data file used to convert GMT (UTC) to mlocal solar time ********************
time_corr_fn = '../t5s_corr.dat' ;'t5n_corr.dat'          ;;arctic
ncols = 1605 & nrows = 1605  & time_corr = fltarr(ncols,nrows)
openr,ur,time_corr_fn,/get_lun
      readu,ur,time_corr
close,ur
;************ convert GMT (UTC) to mlocal solar time ********************
dimxyS = 1605
img1 = intarr(dimxyS,dimxyS,nfields)
img2 = intarr(dimxyS,dimxyS,nfields)
score1 = fltarr(dimxyS,dimxyS) + 100.   ; Scan-time score. 0=best
score2 = fltarr(dimxyS,dimxyS) + 100.   ; Scan-time score. 0=best
satjday = 0
hour_start = 0
hour_end   = 0

iwhere = where(strmatch(flns,'*.S1') eq 1,count)  ; set 1
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end
  if hour_end gt hour_start then flag=0 else flag=1  
  
  ncols = dimxyS
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  ;drop_in,hemis,ncols,nrows,img,s12z,s12zscore,lst,hourdiff,flag
  drop_in,img,img1,score1,lst_an1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_an2,hourdiff,flag,time_corr
endfor

iwhere = where(strmatch(flns,'*.S2') eq 1,count)  ; set 2
if count gt 0 then flns2 = flns[iwhere]
nflns = n_elements(flns2)

for i=0,nflns-1 do begin 
  fln = flns2[i]
  print,'Processing file: ', fln
  
  reads,strmid(fln,str_pos1,str_len1),satjday
  hourdiff = (satjday-jday)*24.
  
  ;--------------------------------------------------------------------------
  if jday eq 1 and (satjday eq 365 or satjday eq 366) then hourdiff = -24.
  if (jday eq 365 or jday eq 366) and satjday eq 1 then hourdiff = 24.
  ;--------------------------------------------------------------------------
  
  reads,strmid(fln,str_pos2,str_len2),hour_start
  reads,strmid(fln,str_pos3,str_len3),hour_end
  if hour_end gt hour_start then flag=0 else flag=1
  
  ncols = dimxyS
  nrows = ncols
  read_merge,fln,ncols,nrows,img,use_ch3a
  ;drop_in,hemis,ncols,nrows,img,s12z,s12zscore,lst,hourdiff,flag
  drop_in,img,img1,score1,lst_an1,hourdiff,flag,time_corr
  drop_in,img,img2,score2,lst_an2,hourdiff,flag,time_corr
endfor
print,'  Writing the composite...'
;  WRITE the composites.
openw,u,Sfile1,/get_lun
writeu,u,img1
free_lun,u

openw,u,Sfile2,/get_lun
writeu,u,img2
free_lun,u


print,'Done compositing.'
end


;************************** READ AND MERGE THE AVHRR DATA *******************

pro read_merge,fln,ncols,nrows,imageall,use_ch3a
;----------------------------------------------------------------------------
;  Reads and merges the CCAR AVHRR GAC data into a single array, and does
;  some conversions.
;
;  The input data type is UNSIGNED short (2-byte) integer with units
;  as described below for the output format.
;
;  The new format is:
;     CH1 % ALBEDO * 10, NORMALIZED BY THE COSINE OF THE SOLAR ZEN. ANG.
;     CH2 % ALBEDO * 10, NORMALIZED BY THE COSINE OF THE SOLAR ZEN. ANG.
;     CH3 TB * 10 and/or % ALBEDO *10.  For NOAA-16 and higher, the sunlit portion
;       of channel 3A will be normalized by the cosine of the solar zenith angle;
;       the dark portion will remain as Tbs*10..
;     CH4 TB * 10
;     CH5 TB * 10
;     SCAN ANGLE (DEGREES) * 10
;     SOLAR ZENITH ANGLE (DEGREES) * 10
;     RELATIVE AZIMUTH ANGLE (DEGREES) * 10
;     TIME (GMT) * 10
;  All output values are SIGNED short (2-byte) integer.
;----------------------------------------------------------------------------

;print,'Working on ', fln

nfields = 10
field=strarr(nfields)
field(0)=fln+'.1'
field(1)=fln+'.2'
field(2)=fln+'.3'
field(3)=fln+'.4'
field(4)=fln+'.5'
field(5)=fln+'.ELEV'
field(6)=fln+'.SZA'
field(7)=fln+'.AZIM'
field(8)=fln+'.SAZA'
field(9)=fln+'.TIME'

;  READ the data for fields 0-6. 

imageall = intarr(ncols,nrows,nfields-1)
labels = ['ch1','ch2','ch3','ch4','ch5','Scan angle','Solar zenith angle']

for i=0,6 do begin
  image = uintarr(ncols,nrows)
  openr,u,field[i],/get_lun
  readu,u,image
  close,u
  free_lun,u
  
  if (!version.arch ne 'ppc') then image = swap_endian(image)
  convertint,image,labels[i]

; DETERMINE SCAN ANGLE

  if (i eq 5) then begin
;    print,'  Converting to scan angle...'
    elev2scan,image
    image = fix(image)
  endif
    
  imageall(*,*,i) = image(*,*)
  
;  print,"Min/max in image ",i,": ",min(image), max(image)
endfor

; NORMALIZE CHANNELS 1, 2, and 3A BY THE COSINE OF THE SOLAR ZENITH ANGLE
; After discussion with Xuanji, he suggested not do the normalization here,
; wait until CASPR.

;print,'  Normalizing channels 1, 2, and 3A...'
;normalize,imageall,0,use_ch3a
;normalize,imageall,1,use_ch3a
;if use_ch3a then normalize,imageall,2,use_ch3a

; DETERMINE RELATIVE AZIMUTH

relaz,field[7],field[8],ncols,nrows,image
imageall(*,*,7) = image(*,*)

; Read the TIME image

image = uintarr(ncols,nrows)
openr,u,field[9],/get_lun
readu,u,image
free_lun,u
if (!version.arch ne 'ppc') then image = swap_endian(image)
convertint,image,'Time'
imageall(*,*,8) = image(*,*)

return
end

;************************* DROP AN IMAGE INTO THE COMPOSITE ****************

pro drop_in,img,img12z,oldscore,lst,hourdiff,flag,time_corr,utc=utc
;---------------------------------------------------------------------------
; Adds an image to the local solar time lst composite, depending on the scan angles
; and time of acquisition.
;
; Input                                                   
;   img     : The image to be added (ncols x nrows x 9; see 'read_merge')
;   img12z  : The 12Z composite image (ncols x nrows x 8; no time item)
;   oldscore: The previously computed "score" for each pixel based on the
;             scan angle and time (ncols x nrows) [fltarr]
;   hourdiff: added to account the hour difference from previous and next day
;   time_corr: data files used to convert UTC to Local Solar Time, 
;              this file is generated based on longitude of grid
;   UTC:      keyword, if set, the composite is done at set UTC,
;             otherwise at set Local Solar Time                                                 
; Output                                             
;   img12z  : The local solar time lst composite image, same size as 'img'.  
;              (ncols x nrows x 8) [intarr]
;   oldscore: The new scores for those pixels that had lower scores than
;             before.  Otherwise the pixel scores are unchanged.
;             (ncols x nrows) [fltarr], here it is satellite scan angle
;---------------------------------------------------------------------------
TARGETTIME = lst        ; Target local solar time, or UTC is keyword UTC set
MAXTDIFF = 3.0           ; Maximum time difference from target time allowed
;SCANWGT = 0.4           ; Weight for scan angle in score function
;TIMEWGT = 0.6           ; Weight for time in score function
;---------------------------------------------------------------------------

; Determine which part of the input image is not filled.  Check channel 4.
; The nav/cal programs fill the rest of the image with the minimum value
; within the valid part (?).  Looking for a time of 0 probably gets it
; all anyway.  'badpix' will have values of 0 for good pixels  and 1 for
; bad pixels.

;badpix = bytarr(ncols,nrows)*byte(0)
;ibad = where(img(*,*,3) eq min(img(*,*,3)),count)
;if count gt 0 then badpix(ibad) = 1
;ibad = where(img(*,*,3) lt 1500 or img(*,*,3) gt 4000 or img(*,*,8) eq 0,count)
badpix = byte(oldscore)*byte(0)
ibad = where(img(*,*,3) lt 1500 or img(*,*,3) gt 4000, count)
if count gt 0 then badpix(ibad) = 1

igood = where(badpix eq 0,count)
if count eq 0 then begin
  print,'DROP_IN: NO GOOD PIXELS.  THE IMAGE WILL BE SKIPPED.'
  return
end

; Get the average time and determine if it is close enough in time to the
; composite to be used. Don't use the image if its time is more than MAXTDIFF
; hours before or after the target time.  Compute a scoring function based
; on the scan angle and time from target time (lower score is better), then
; replace the previous image values for those pixels with lower scores
; than before.

time = img(*,*,8)          ; time is input as hhmms, where s=seconds/10.
hhmm = fix(time/10.)
hh = fix(hhmm/100.)
mm = hhmm - hh*100
ss = (time/10. - hhmm) * 100.
dhr = hh + mm/60. + ss/3600.

;;; some granules have values larger than 24 for the observations 
;;; from next day, e.g. starting time is current day, but ending time is 
;;; the day after. these values are subtracted by 24
xxx = where(dhr gt 24., cxx)
if cxx gt 0 then dhr(xxx) = dhr(xxx)-24. 
;;; adjustment for that starting and ending day are different
if flag eq 1 then begin
   xxx = where(dhr gt 0. and dhr lt 12., cxx)
   if cxx gt 0 then dhr(xxx) = dhr(xxx) + 24.
endif
;;; adjustment for day changes
dhr = dhr + hourdiff

;************ convert GMT (UTC) to mlocal solar time ********************
if keyword_set(utc) eq 0 then  dhr = dhr + time_corr
;************ convert GMT (UTC) to mlocal solar time ********************


  scan = reform(img(*,*,5)/10.)
  ;x = where(scan(igood) gt 60., cx)
   
  ;tdiff1 = abs( dhr - TARGETTIME + 24)
  ;tdiff2 = abs( dhr - TARGETTIME + 0)
  ;tdiff3 = abs( dhr - TARGETTIME - 24)
  ;tdiff  = tdiff1<tdiff2<tdiff3    
  ;ibetter = where(tdiff lt MAXTDIFF and scan lt oldscore and badpix eq 0,count)

  tdiff = abs( dhr - TARGETTIME + 0)
  ibetter = where(tdiff lt MAXTDIFF and scan lt oldscore and badpix eq 0,count)
  
  if count gt 0 then begin
    print,'  Dropping image into composite...'

    for i=0,8 do begin
      tmpcomp = img12z(*,*,i)
      tmpimg = img(*,*,i)
      tmpcomp(ibetter) = tmpimg(ibetter)
      img12z(*,*,i) = tmpcomp(*,*)
    endfor
    oldscore(ibetter) = scan(ibetter)
  endif else begin
    print,'  This image will not be used (not any better than current composite).'
  endelse

end


;*************************** CONVERT TO SCAN ANGLE *************************

pro elev2scan,image
;---------------------------------------------------------------------------
; CONVERTS SATELLITE ELEVATION ANGLE TO SCAN ANGLE
;
; Input                                                   
;   eang   :  The satellite elevation angle (degrees * 10)
;                                                    
; Output                                             
;   scan   :  The satellite scan angle (degrees * 10)
;
; Re is the radius of the earth, Hsat is the nominal height of the
; satellite, and ReHsat = Re + Hsat
;---------------------------------------------------------------------------

eang = image/10.
degrad = 0.017453292 
Rearth = 6357. 
Hsat = 850. 
ReHsat = 7207.0 
zenang = (90. - eang) * degrad 
scan = asin(Rearth*sin(zenang)/(ReHsat)) / degrad 
image = fix(scan*10.)

return
end 


;************************* RELATIVE AZIMUTH ANGLE **************************

pro relaz,satazfln,solazfln,ncols,nrows,image
;---------------------------------------------------------------------------
; DETERMINE THE RELATIVE AZIMUTH BY SIMPLY SUBTRACTING THE SOLAR
; AZIMUTH FROM THE SATELLITE AZIMUTH.
;
; INPUT: 
;   satazfln, 
;   solazfln   : File names for the satellite and solar azimuth angles
;   ncols,
;   nrows      : The size of the image
; 
; OUTPUT: 
;   image      : Integer array of relative azimuth values
;---------------------------------------------------------------------------

imagea = uintarr(ncols,nrows)
openr,u,satazfln,/get_lun
readu,u,imagea
free_lun,u
if (!version.arch ne 'ppc') then imagea = swap_endian(imagea)
convertint,imagea,'Satellite azimuth'

imageb = uintarr(ncols,nrows)
openr,u,solazfln,/get_lun
readu,u,imageb
free_lun,u
if (!version.arch ne 'ppc') then imageb = swap_endian(imageb)
convertint,imageb,'Solar azimuth'

image = abs(imagea - imageb)
ihi = where(image gt 1800,count)
if count gt 0 then image(ihi) = 3600 - image(ihi)

return
end


;******************************* NORMALIZE *******************************

pro normalize,imgall,bandnum,use_ch3a
;---------------------------------------------------------------------------
; Normalize a shortwave channel by the cosine of solar zen angle.
; The values in the original array are replaced.  It is assumed
; that the solar zenith angle is in imgall(*,*,6).
;---------------------------------------------------------------------------

img = imgall(*,*,bandnum)
zen = imgall(*,*,6)/10.

; If this is channel 3A only normalize the sunlit portions of the image.
; Otherwise normalize the whole image.

if bandnum eq 2 then begin
  if use_ch3a then begin
    ilit = where(zen lt 90,count)
    if count gt 0 then begin
      img(ilit) = fix(((img(ilit)/10.) / cos(zen(ilit)*0.017453)) * 10.)
    endif
  endif else begin
    return                      ;  If channel 3B was passed in, return.
  endelse
endif else begin
  img = fix(((img/10.) / cos(zen*0.017453)) * 10.)
endelse

imgall(*,*,bandnum) = img

end


;*************************** INTEGER SIZE CHECK ****************************

pro convertint,image,label
;----------------------------------------------------------------------------
; Convert unsigned integer to signed integer (both 2 bytes), check for 
; values too large for signed integer.  If more than 1/10 of 1% are
; bad then the procedure is aborted.  Otherwise, the bad pixels are
; reset to 0 and processing continues.
;
; Input:
;   image  : The image to be converted
;   label  : A string identifying what's being converted.  Used if there's
;            an error.
;----------------------------------------------------------------------------

ibig = where(image gt 32767,count)
;if count gt 0 then begin
;  if n_elements(label) eq 0 then label = ''
;  print,'>>> CONVERTINT: VALUES ARE TOO LARGE FOR SIGNED INTEGER CONVERSION <<<'
;  print,'    Type: ',label,', number of pixels: ',count,'
;  print,'    Min and max values in image: ',min(image),max(image)
;  nmaxbad = n_elements(image) * 0.001       ; 1/10 of 1% are allowed to be bad
;  if count gt nmaxbad then begin
;    print,'    Too many bad pixels.  Cannot continue.'
;    exit
;  endif else begin
;    print,'    Bad pixels will be set to 0.'
;    image(ibig) = 0
;  endelse
;endif

if count gt 0 then begin
   image(ibig) = 0
endif

image = fix(image)

end


;************************* SUBSAMPLE AN IMAGE ******************************
;[ch3a_active]

function ch3a_active, satnum,year,month,day
;---------------------------------------------------------------------------
;  Determines if a satellite was using AVHRR channel 3A (1.6 um) during the
;  sunlit portion of the day based on  the satellite number and date. 
;  For example, NOAA-16 switched between 3A and 3B until May 1, 2003. 
;
;  Input:
;    satnum       : The NOAA satellite number (e.g., 7, 9, 11, 12, 14, 15, 16).
;    year, month,
;    day          : 2- or 4-digit integer for the year, 2-digit for month, and day.
;                   If year is 2-digit, values >=50 are 19xx, values <50 are 20xx.
;    hour         : The hour on a 24-hour clock, GMT (floating point)
;
;  Output:
;    TRUE if channel 3A was used; FALSE otherwise.
;
;  Notes:
;  1. Channel 3A on NOAA-15 was only used briefly, so we assume that 
;     it's always the 3.7 um channel.  This could/should be changed here.
;  2. As of May 1, 2003, NOAA-16 uses channel 3b (3.7 um) at all times.

;  Called by: ingest_image
;
;  Calls: None
;
;[enddoc]
;---------------------------------------------------------------------------
TRUE = 1
FALSE = 0
;---------------------------------------------------------------------------

if (satnum eq 16 and year eq 2003 and month lt 5) or $
   (satnum eq 16 and year le 2002) or $
   (satnum eq 18 and year eq 2005 and month eq 7) or $
   (satnum eq 18 and year eq 2005 and month eq 8 and day le 5) then begin
  ch3a = TRUE
  print,'  Assuming that channel 3 is 1.6 um for sunlit areas.'
endif else begin
  ch3a = FALSE
  print,'  Assuming that channel 3 is 3.7 um at all times.'
endelse

return,ch3a
end
