   ;-------------------------------------------------------------------------------------- 
   ;read_gac_forAPP.pro    2015-08-14  Yinghui Liu
   ;-------------------------------------------------------------------------------------- 
   ; AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   ; NAME: read_gac_forAPP.pro 
   ; PURPOSE: extract, calibrate, and navigate AVHRR GAC data
   ; DESCRIPTION: read AVHRR Global Area Coverage (GAC) data, calibrate the data using
   ;              Andrew Heidinger (Andrew.Heidinger@noaa.gov) calibration and navigation
   ;              procedure,remap the calibrated data with the MODIS Swath-to-Grid (MS2GT).
   ;              The calibration approach is developed by Andy Heidinger. It was in Fortran
   ;              language, and converted to IDL language by the authors. 
   ;
   ; AUTHORS:
   ;  Yinghui Liu, yinghuil@ssec.wisc.edu
   ;  Richard Dworak, rdworak@ssec.wisc.edu
   ;  Jeffrey Key, jeff.key@noaa.gov
 
   ; COPYRIGHT  
   ; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   ; DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   ; FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   ; INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   ; EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   ; DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   ; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   ; SUPPORT TO USERS.
   
   ; Software required
   ; IDL: Interactive Data Language
   ; MS2GT: the MODIS Swath-to-Grid
   
   ; Routines in this script and their purpose:
   ; IDL Subtoutine:
   ;        mreadf.pro  
   ;        viewing_geometry.pro  
   ;        Plank_AVHRR.pro  
   ;        reposnx.pro  
   ;        AVHRR_Calibration_APP.pro
   ;        avhrr_caligration_subs_tool.pro  
   ;        avhrr_caligration_subs_0.pro  
   ;        avhrr_caligration_subs_5.pro  
   ;        avhrr_caligration_subs_1.pro  
   ;        avhrr_caligration_subs_2.pro  
   ;        avhrr_caligration_subs_3.pro  
   ;        avhrr_caligration_subs_4.pro  
   ;   
   ;
   ; input:
   ;  filename: AVHRR GAC file name
   ; outputs:
   ;  (BT: Brightness Temperature)
   ;  filenaem.N1.1: ch1 reflecance of first part of the overpss before passing north pole point in the Northen Hemisphere
   ;  filenaem.N1.2: ch2 reflecance of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.3: ch3a reflecance/ch3b BT of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.4: ch4 BT of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.5: ch5 BT of first part of the overpss before passing north pole point in the NH
   ;  filenaem.N1.SZA: solar zenith angle in the NH
   ;  filenaem.N1.ELEV: sensor elevation angle in the NH
   ;  filenaem.N1.AZIM: sensor relative azimuth angle in the NH
   ;  filenaem.N1.TIME: time of observation in UTC in the NH
   ;  filenaem.N1.SAZA: satellite relative azimuth angle in the NH
   ;  filenaem.N2.* :  second part of the overpss after passing north pole point in the NH
   ;  filenaem.S1.* : first part of the overpss before passing south pole point in the Sourthern Hemisphere 
   ;  filenaem.S2.* : second part of the overpss after passing south pole point in the Sourthern Hemisphere 
   -------------------------------------------------------------------------------------- 





PRO WRITE_BINARY, FILE, DATA, APPEND=APPEND
if (n_elements(append) eq 0) then append = 0
openw, lun, file, /get_lun, append=append
writeu, lun, data
free_lun, lun
END


pro lonto360,lonn

xxx = where(finite(lonn) and lonn lt 0., cxx)
if cxx gt 0 then lonn(xxx) = lonn(xxx)+360.
end

pro lonto180,lonn

xxx = where(finite(lonn) and lonn gt 180., cxx)
if cxx gt 0 then lonn(xxx) = lonn(xxx)-360.
end


;---------------------------------prepare for APP----------------------------------------
pro read_gac_forAPP, filename



;;used to be
;read_gac_n18_1, filename,avhrr_data, lat, lon, szaa, satz, relaz, utc, $
;                  year=year, day=day, second=second 

;; after adapt Andy's approach

AVHRR_Calibration_APP,filename,avhrr_data, lat, lon, szaa, satz, relaz, utc

fn=strmid(filename,strpos(filename,'/',/reverse_search)+1)
;print,fn
sat_elev = 90. - satz   ;;; convert satellite zenith angle to satellite elevation angle, elevation angle = 90. - sensor_zenith_angle, confirmed
dims = size(utc,/dimension)
utc_app = intarr(dims(0),dims(1))
utc1 = fix(utc/3600.)       ;;; hour
utc2 = fix((utc-utc1*3600.)/60.)   ;; minute
utc3 = fix((utc-utc1*3600.-utc2*60)/10.)   ;; second/10.
utc_app = utc1*1000+utc2*10+utc3



nfields = 11
datat = fltarr(nfields,dims(0),dims(1))
datat(0,*,*) = lat
datat(1,*,*) = lon
datat(2:6,*,*)=avhrr_data
datat(7,*,*) = szaa
datat(8,*,*) = sat_elev
datat(9,*,*) = relaz        
datat(10,*,*) = utc_app
;;;; visible reflectance are in %, with all values for solar zenith angle less than 90 degree positive
;;;; brightness temperature for ch3,4,5 have negative values (same value), see README

nx     = 409     ;;; dims(0)
nlines = dims(1)
nout = lonarr(4)

if nlines gt 0 then begin

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; extract data ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   northlim =  30.   ;; 30 degree north
   southlim = -30.   ;; 30 degree south

   writeflag=0
   northin=0
   southin=0
   wtog=0
   northfile = 0
   southfile = 0
   northbound= 90.
   southbound = -90. 
   southstart = 0 ;south hemi start flag
   northstart = 0 ;north hemi start flag
   skip_count= 0. ;missing line counter
      
   mid_index = 204  ;; middle of 409
   
   
   datat_NS = fltarr(4,nfields,nx,nlines)
   
   for ic=0,nlines-1 do begin
      if writeflag eq 0 then wtog = 0
      
      lat0 = datat(0,mid_index,ic)                         ;;; latitude of subpoint of the satelliet 

      if lat0 gt northlim and lat0 le northbound then begin
         northin = 1
	 southstart = 0
	 skip_count = 0
	 if wtog eq 0 or northstart eq 0 then begin
	    northstart = 1
	    if northfile eq 0 then begin
	       wtog = 1
	       nout(0) = 0
	       northfile = 1
	    endif else begin
	       wtog = 1
	       nout(1) = 0
	       northfile = 2
	    endelse
	 endif   
	 
	 if northfile eq 1 then begin
	    datat_NS(0,*,*,nout(0)) = datat(*,*,ic)
	    nout(0) = nout(0)+1
	    writeflag = 1
	 endif
	 if northfile eq 2 then begin
	    datat_NS(1,*,*,nout(1)) = datat(*,*,ic)
	    nout(1) = nout(1)+1
	    writeflag = 1
	 endif	
	  
      endif else begin
         northin = 0
      endelse 
	 
      
      if lat0 lt southlim and lat0 ge southbound then begin
         southin = 1
	 northstart = 0
	 skip_count=0
	 if wtog eq 0 or southstart eq 0 then begin
	   southstart = 1
	   if southfile eq 0 then begin
	       wtog = 1
	       nout(2) = 0
	       southfile = 1
	   endif else begin
	       wtog = 1
	       nout(3) = 0
	       southfile = 2
	   endelse
	 endif
	 
	 if southfile eq 1 then begin
	    datat_NS(2,*,*,nout(2)) = datat(*,*,ic)
	    nout(2) = nout(2)+1
	    writeflag = 1
	 endif
	 if southfile eq 2 then begin
	    datat_NS(3,*,*,nout(3)) = datat(*,*,ic)
	    nout(3) = nout(3)+1
	    writeflag = 1
	 endif
	 
      endif else begin
         southin = 0
      endelse
      
      if northin eq 0 and southin eq 0 then begin
      skip_count = skip_count+1 
       if skip_count ge 100 then writeflag = 0 
      endif 
      
   endfor     	  
   
   area_title = ['N1','N2','S1','S2']
   data_title = ['1','2','3','4','5','SZA','ELEV','AZIM','TIME']
   cam_per_scan = 10
   const1 = !pi/180.
   grid_sizes = [1805,1805,1605,1605]
   gpdfiles =['Na5.gpd','Na5.gpd','Sa5.gpd','Sa5.gpd']
   
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; reproject the data ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   for ic=0,4-1 do begin
      if nout(ic) gt 1 then begin
         latt = reform(datat_NS(ic,0,*,0:nout(ic)-1))
         lonn = reform(datat_NS(ic,1,*,0:nout(ic)-1))
         dima = size(lonn,/dimension)
         xsize = dima(0)
         ysize = dima(1)/cam_per_scan
         latt = latt(*,0:ysize*cam_per_scan-1)  & lonn = lonn(*,0:ysize*cam_per_scan-1)  
         append = 0
         write_binary,'lat.dat',latt,append=append
         write_binary,'lon.dat',lonn,append=append
   
   
         str1 = string(xsize)  & str2 = string(ysize)  & nnn = string(cam_per_scan)  & tag = 'avhrr'
         stra = string(xsize,format='(i05)')  & strb = string(ysize,format='(i05)')  & ttt = string(cam_per_scan,format='(i02)')
         comm1 = 'll2cr  -f '+ string(xsize,format='(i5)')+string(ysize,format='(i5)')+'  '+ttt+' lat.dat lon.dat '+gpdfiles(ic)+' ' +tag
	 spawn,comm1
	 	 	 
	 grid_size = grid_sizes(ic)
	 if ic eq 0 or ic eq 1 then str_ns = ' 1805 1805 '  else str_ns =' 1605 1605 '
	 
	 field_count = 0
	    
         for id=0,nfields-2-1 do begin
	    temp = reform(datat_NS(ic,id+2,*,0:nout(ic)-1))               ;;; reflectance are set to -999. for solar zenith angle larger than 90
	    if id eq 0 or id eq 1 then begin
	       temp_szaa = reform(datat_NS(ic,7,*,0:nout(ic)-1))          ;;;;solar zenith angle 
	       xxx = where(temp_szaa ge 90., cxx)
	       if cxx gt 0 then temp(xxx) = -999.
	       ;xxx = where(temp_szaa gt 0. and temp_szaa lt 90., cxx)     ;;;;; test, normalize the reflectance with solar zenith angle
	       ;if cxx gt 0 then temp(xxx) = temp(xxx)/cos(temp_szaa(xxx)*const1)
	    endif   
	    if id eq 2 or id eq 3 or id eq 4 then begin                 ;;; brightness temperature less than 0. are set to -999. for remap
	       xxx = where(temp lt 0., cxx)
	       if cxx gt 0 then temp(xxx) = -999.
	    endif   
	    
	    temp = temp(*,0:ysize*cam_per_scan-1)	    
            write_binary,'temp.dat',temp,append=append

            name1 = tag+'_cols_'+stra+'_'+strb+'_00000_'+ttt+'.img'
            name2 = tag+'_rows_'+stra+'_'+strb+'_00000_'+ttt+'.img'
	    
            comm2 = 'fornav 1 -t f4 -f -999. -F -999. -D 5.0 -m '+str1+str2+nnn+' '+name1+' '+name2+' '+'temp.dat '+ str_ns +fn+'.'+area_title(ic)+'.'+data_title(id)+'a'
            spawn,comm2
	    
	    
	    file_res = file_info(fn+'.'+area_title(ic)+'.'+data_title(id)+'a')
	    if file_res.exists eq 1 then begin	    
	       dtemp1 = fltarr(grid_size,grid_size)
	       dtemp2 = uintarr(grid_size,grid_size)
	       openr,ur,fn+'.'+area_title(ic)+'.'+data_title(id)+'a',/get_lun
	          readu,ur,dtemp1
	       free_lun,ur
	    
	       if id ne nfields-2-1 then begin
	          dtemp2 = uint(abs(dtemp1)*10.)                  ;;; abs is for relative azimuth angle (-180, 180)
	       endif else begin
	          dtemp2 = uint(abs(dtemp1)*1.)
	       endelse     
	    
	       xxx = where(dtemp1 lt -200. , cxx)             ;;; acutally -999., 
	       if cxx gt 0 then dtemp2(xxx) = uint(0)            ;;; for relative azimuth angle, set to 0 too
	    
	       ;;;;;;;;;;;;;;;;;;; note, this is for consistence with composite_lst.pro ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
               if (!version.arch ne 'ppc') then dtemp2 = swap_endian(dtemp2)
	    
	       openw,uw,fn+'.'+area_title(ic)+'.'+data_title(id),/get_lun
	          writeu,uw,dtemp2
	       free_lun,uw
	    
	       spawn,'rm '+fn+'.'+area_title(ic)+'.'+data_title(id)+'a'
	       
	       field_count = field_count+1
	    endif
	    
         endfor	 
	 
	 if field_count eq nfields-2  then begin
	 
	    ;;;;; for composite_lst.pro also
	    dtemp2 = uintarr(grid_size,grid_size)*0
            if (!version.arch ne 'ppc') then dtemp2 = swap_endian(dtemp2)
	    openw,uw,fn+'.'+area_title(ic)+'.SAZA',/get_lun
	       writeu,uw,dtemp2
	    free_lun,uw	 	 	 
	 endif else begin
	    spawn,'rm '+fn+'.'+area_title(ic)+'*'
	 endelse   
	  
	 spawn,'rm *.img *.dat'
      endif
   endfor	    
   
   	    	    	    
endif



end

