#!/bin/sh
   #######################################################################################
   # run_APP.sh   2015-06-01  Yinghui Liu
   #-------------------------------------------------------------------------------------- 
   # AVHRR Polar Pathfinder (APP) PROCESSING SOFTWARE Version 1.1
   # NAME: run_APP.sh
   # PURPOSE: Generate APP from Jan 1st 1982 to the present
   # DESCRIPTION: read AVHRR Global Area Coverage (GAC) data, calibrate the data using
   #              Andrew Heidinger (Andrew.Heidinger@noaa.gov) calibration and navigation
   #              procedure,remap the calibrated data with the MODIS Swath-to-Grid (MS2GT),
   #              and composite the daily APP data.
   #
   # AUTHORS:
   #  Yinghui Liu, yinghuil@ssec.wisc.edu
   #  Richard Dworak, rdworak@ssec.wisc.edu
   #  Jeffrey Key, jeff.key@noaa.gov
   #
   # COPYRIGHT  
   # THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
   # DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
   # FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
   # INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
   # EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
   # DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
   # THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
   # SUPPORT TO USERS.
   
   # Software required
   # IDL: Interactive Data Language
   # MS2GT: the MODIS Swath-to-Grid
   
   # Routines in this script and their purpose:
   # Shell Script Subroutine:
   #
   # set_env: set Environmental Variable
   # get_julianday_year: get the julian day and year of the previous day and the day after
   # cp_data_from_fjord: function to get the AVHRR GAC data (NEED MODIFICATION for YOUR
   #                     OWN PURPOSE)
   # extract_calibrate: uses the Andrew Heidinger's procedure used in CLAVR-x to process
   #                    the AVHRR GAC data. 
   # cp_from_data_to_composite: move the calibrated data from data directory to 
   #                            composite directory
   # get_satnum: get the satnum on the current day in current year
   # rm_data: remove the calibrated data in directory: data
   #
   # IDL Subtoutine:
   #   to read, extract, and calibrate the AVHRR GAC data
   #        mreadf.pro  
   #        avhrr_caligration_subs_tool.pro  
   #        viewing_geometry.pro  
   #        avhrr_caligration_subs_0.pro  
   #        Plank_AVHRR.pro  
   #        reposnx.pro  
   #        avhrr_caligration_subs_5.pro  
   #        avhrr_caligration_subs_1.pro  
   #        avhrr_caligration_subs_2.pro  
   #        avhrr_caligration_subs_3.pro  
   #        avhrr_caligration_subs_4.pro  
   #        AVHRR_Calibration_APP.pro  
   #        read_gac_forAPP.pro 
   #        ext_cal_nav.pro 
   #
   #   to fill in the daily composite
   #        composite_lst.pro
   #
   # Inputs:
   #    1. for daily APP composite, AVHRR GAC data from previous day, current day, and the 
   #       day after is needed. These GAC data is available from the National Oceanic 
   #       and Atmospheric Administration (NOAA) the Comprehensive Large Array-data Stewardship 
   #       System (CLASS), www.class.noaa.gov. The user can order the data from the NOAA CLASS 
   #       for free. The steps to order the data from NOAA CLASS is described in README.
   #       In this case, you should get the GAC data and put them in $data_dir before running
   #       run_APP.sh, or modify cp_data_from_fjord() for your own purpose.
   #    2. To calibrate the AVHHR Level 1b GAC data, file avhrr_##_instr.dat in the ./Ancil_Data_Dir
   #       storing constants for calibration is needed, in which ## represents the satellite number.
   #       Those files are from Andrew Heidinger (Andrew.Heidinger@noaa.gov). Updated of these 
   #       files may be broadcasted by NCDC.
   #    3. Surface type mask is provided by the authors, and will be updated routinely.
   #Outputs:
   #       Outputs of this program include APP daily composites at 2 local solar time for each pole,
   #       Arctic and Antarctic in directory ${result_dir}/${year_curr}
   #-------------------------------------------------------------------------------------- 
   
   . ./myfuncs
   
   # set the environmental variable
   set_env
   definedirs      
      
   #  set up the directories needed
   if [ ! -d ${data_dir} ] ; then
      mkdir ${data_dir}
   fi
   
   if [ ! -d ${composite_dir} ] ; then
      mkdir ${composite_dir}
   fi
   
   if [ ! -d ${result_dir} ] ; then
      mkdir ${result_dir}
   fi
   
   # set the starting date and end date of APP
   year_start=1982
   day_start=1
   year_end=2014
   day_end=365
   
   year_curr=${year_start}
   
    while (( $year_curr <= $year_end )) ; do
       if (( $year_curr == $year_start )) ; then
         day_curr=$day_start
      else
         day_curr=1
      fi
      if (( $year_curr % 4 == 0 )) ; then
          end_day=366
      else
         end_day=365
      fi
      
      if (( $year_curr == $year_end )) ; then
         end_day=$day_end
      fi 
      
      result_year=${result_dir}'/'${year_curr}
      if [ ! -d ${result_year} ] ; then
         mkdir ${result_year}
      fi
            
      while (( $day_curr <= $end_day )) ; do 
         ## get the julian day and year of current, previous day and the data after 
         get_julianday_year $day_curr $year_curr day_prev year_prev day_aft year_aft   
	 
	 if (( $year_curr == $year_start && $day_curr == $day_start )); then
	   ## get the AVHRR GAC data (NEED MODIFICATION for YOUR OWN PURPOSE)
           cp_data_from_fjord $day_prev $year_prev
           cp_data_from_fjord $day_curr $year_curr
	   ##  read, extract, and calibrate the AVHRR GAC data
	   extract_calibrate  $day_prev $year_prev
           extract_calibrate  $day_curr $year_curr 
	 fi
	    
	 ## get the AVHRR GAC data (NEED MODIFICATION for YOUR OWN PURPOSE)
         cp_data_from_fjord $day_aft $year_aft
	 ##  read, extract, and calibrate the AVHRR GAC data
	 extract_calibrate  $day_aft $year_aft   
	 	 
	 cp_from_data_to_composite  $day_prev $year_prev -1 
	 cp_from_data_to_composite  $day_curr $year_curr 0
	 cp_from_data_to_composite  $day_aft $year_aft 1 
	 
	 #fill in the NPP daily composites, 
	 # and generate and move the output to ${result_dir}/${year_curr}
	 composite ${year_curr} ${day_curr}
	 
	 #remove the data no longer needed
         rm_data  $day_prev $year_prev
	 
         ((day_curr = day_curr + 1))
      done
      ((year_curr = year_curr+1))	 
   done
   
   
   
   
